/********************************************************************
*********************************************************************
*	Processor		: PIC18F2550
*	FileName		: HardwareProfile - PIC18F2550.h
*	Dependencies	: See INCLUDES section
*	Compiler		: Microchip C18
*	USB Framework	Microchip Application Libraries v2009-08-31.EXE
*		( for PIC18)
*	Reference source: HardwareProfile - PIC18F Starter Kit 1.h  etc.
*
*	History:
*	Date		Designer	Description
*	07/23/2010	DachsTulip	Created	
********************************************************************
********************************************************************/

/**
********************************************************************
	PIC18F2550dl
		vOTCY	F32KoCg
		RAMTCY				F2048oCg
		I/O					F24
		pbP[W				F28sDIP
		x					F48MHz
		EEPROM					F256oCg
		f[^[Ro[^		FA/D 10ch x 10bit
		C^[tF[X		F 1-A/E/USART, 1-MSSP(SPI/I2C), USB
********************************************************************
	*/

#ifndef  HARDWARE_PROFILE_PIC18F2550_H
#define HARDWARE_PROFILE_PIC18F2550_H

#include	"mytype.h"

    /*******************************************************************/
    /******** USB stack hardware selection options *********************/
    /*******************************************************************/
    //This section is the set of definitions required by the MCHPFSUSB
    //  framework.  These definitions tell the firmware what mode it is
    //  running in, and where it can find the results to some information
    //  that the stack needs.
    //These definitions are required by every application developed with
    //  this revision of the MCHPFSUSB framework.  Please review each
    //  option carefully and determine which options are desired/required
    //  for your application.

    //The PICDEM FS USB Demo Board platform supports the USE_SELF_POWER_SENSE_IO
    //and USE_USB_BUS_SENSE_IO features.  Uncomment the below line(s) if
    //it is desireable to use one or both of the features.
    //#define USE_SELF_POWER_SENSE_IO
    // **Note** ȉ́ARpCXCb`"USE_SELF_POWER_SENSE_IO"Ȃ̂Ŏ肠̂܂܂ƂB
    #define tris_self_power     TRISCbits.TRISC2    // Input
	/**
	    #if defined(USE_SELF_POWER_SENSE_IO)
    	    tris_self_power = INPUT_PIN;	// See HardwareProfile.h
    	#endif
    	*/
    #define self_power          1
	/**
		C:\FootMouseKey\system\Usb\usb_device.c
			void USBStdGetStatusHandler(void)
					|-> if(self_power == 1) // self_power is defined in HardwareProfile.h
		*/

    //#define USE_USB_BUS_SENSE_IO
    // **Note** ȉ́ARpCXCb`"USE_USB_BUS_SENSE_IO"Ȃ̂Ŏ肠̂܂܂ƂB
    #define tris_usb_bus_sense  TRISCbits.TRISC2    // Input
	/**
	    #if defined(USE_USB_BUS_SENSE_IO)
    	    tris_usb_bus_sense = INPUT_PIN; // See HardwareProfile.h
    	#endif
		*/
    #define USB_BUS_SENSE       1 
	/**
		C:\FootMouseKey\system\Usb\usb_device.c
			void USBDeviceDetach(void)
					|-> if(USB_BUS_SENSE != 1)
	*/
    
    //Uncomment the following line to make the output HEX of this  
    //  project work with the MCHPUSB Bootloader    
    //#define PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER
	
    //Uncomment the following line to make the output HEX of this 
    //  project work with the HID Bootloader
    #define PROGRAMMABLE_WITH_USB_HID_BOOTLOADER

    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/
    /******** Application specific definitions *************************/
    /*******************************************************************/
    /*******************************************************************/
    /*******************************************************************/

    /** Board definition ***********************************************/
    //These defintions will tell the main() function which board is
    //  currently selected.  This will allow the application to add
    //  the correct configuration bits as wells use the correct
    //  initialization functions for the board.  These defitions are only
    //  required in the stack provided demos.  They are not required in
    //  final application design.
//    #define DEMO_BOARD PIC18F_STARTER_KIT_1
//    #define DEMO_BOARD PICDEM_FS_USB
    #define DEMO_BOARD PIC18F2550

//    #define PICDEM_FS_USB
    #define PIC18F2550					// 2010/04/16()

//    #define CLOCK_FREQ 48000000
    #define CLOCK_FREQ 20000000			// 20MHz
//    #define CLOCK_FREQ 10000000

	#define RA0_RA3_LEDOUT				// LED|[g̎w

	#define ROTARY_DIP_SW_REALCODE		// AER[h`(Rv^ER[h̎̓RgAEg)
										// AER[h => ݒ1ȂABit3=OFF,Bit2=OFF,Bit1=OFF,Bit0=ON
/**
	I/O setting
	C:\Program Files\Microchip\MCC18\h\p18f2550.h
	*/

    /** LED ************************************************************/
	#define mInitAllLEDs()		LATA &= 0xF0; TRISA &= 0xF0;	//LATA=Data Latch register	dataoutput; not affected by analog input.
																//TRISA=Data Direction register
																//TRISA bit (= 0) will make the corresponding PORTA pin an output

	// o͂̓ftH[g"1"INƂȂBOUTɂ"0"ɂKvB2010/10/17() 17:11:40
	#define mInitAllDGSW()		TRISA |= 0x30; TRISC |= 0x07;	//i/o direction setting of (DIG SW + SW1)

	#define mInitEUSART()		TRISC &= ~0x40; TRISC |= 0x80;	//TX(RC6)=OUT RX(RC7)=IN

	#define mRA0_OUT			LATAbits.LATA0
	#define mRA1_OUT			LATAbits.LATA1
	#define mRA2_OUT			LATAbits.LATA2
	#define mRA3_OUT			LATAbits.LATA3

#if defined(RA0_RA3_LEDOUT)

	// 2010/07/16()	LED ON  port=low ONȂ̂ŃCo[^B
	#define mGetLED_1()			!mRA0_OUT
	#define mGetLED_2()			!mRA1_OUT
	#define mGetLED_3()			!mRA2_OUT
	#define mGetLED_4()			!mRA3_OUT

	#if defined(_CIRCUIT_TEST_)
		#define mLED_1_On()
		#define mLED_2_On()
		#define mLED_3_On()
		#define mLED_4_On()

		#define mLED_1_Off()
		#define mLED_2_Off()
		#define mLED_3_Off()
		#define mLED_4_Off()

		#define mLED_1_Toggle()
		#define mLED_2_Toggle()
		#define mLED_3_Toggle()
		#define mLED_4_Toggle()
	#else
		#define mLED_1_On()			mRA0_OUT = 0;
		#define mLED_2_On()			mRA1_OUT = 0;
		#define mLED_3_On()			mRA2_OUT = 0;
		#define mLED_4_On()			mRA3_OUT = 0;

		#define mLED_1_Off()		mRA0_OUT = 1;
		#define mLED_2_Off()		mRA1_OUT = 1;
		#define mLED_3_Off()		mRA2_OUT = 1;
		#define mLED_4_Off()		mRA3_OUT = 1;

		#define mLED_1_Toggle()		mRA0_OUT = !mRA0_OUT;
		#define mLED_2_Toggle()		mRA1_OUT = !mRA1_OUT;
		#define mLED_3_Toggle()		mRA2_OUT = !mRA2_OUT;
		#define mLED_4_Toggle()		mRA3_OUT = !mRA3_OUT;
	#endif			// _CIRCUIT_TEST_

#endif

	/** SWITCH *********************************************************/
	#define mInitAllSwitches()	TRISBbits.TRISB4=1;TRISBbits.TRISB5=1;
	#define FSW_1				PORTBbits.RB5
	#define FSW_2				PORTBbits.RB4
	#define FSW_3				PORTBbits.RB3
	#define FSW_4				PORTBbits.RB2
	#define SW1					PORTCbits.RC2
	#define DGSW_1				PORTAbits.RA4

/**
	timer setting

	C:\Program Files\Microchip\MCC18\h\GenericTypeDefs.h
	C:\Program Files\Microchip\MCC18\h\p18f2550.h
			|-> #define _TON	T1CONbits.TON
			|-> #define _T1IF	IFS0bits.T1IF
	*/

/* INTCON */
#define _GIE		INTCONbits.GIE			// S̊荞݂̋
#define _PEIE		INTCONbits.PEIE			// ӊ荞݂̋

/* PIE1: PERIPHERAL INTERRUPT ENABLE REGISTER 1 */
#define _T1IE		PIE1bits.TMR1IE			// Enables the TMR1 overflow interrupt
/* PIE2: PERIPHERAL INTERRUPT ENABLE REGISTER 2 */
#define _T3IE		PIE2bits.TMR3IE			// Enables the TMR1 overflow interrupt

/* PIR1: PERIPHERAL INTERRUPT REQUEST (FLAG) REGISTER 1 */
#define _T1IF		PIR1bits.TMR1IF			// TMR1 Overflow Interrupt Flag bit
/* PIR2: PERIPHERAL INTERRUPT REQUEST (FLAG) REGISTER 2 */
#define _T3IF		PIR2bits.TMR3IF			// TMR2 Overflow Interrupt Flag bit
											// 1 = TMR1 register overflowed (must be cleared in software)
/* T1CON: TIMER1 CONTROL REGISTER */
#define _T1RUN		T1CONbits.T1RUN			// Timer1 System Clock Status bit
#define _T1CKPS1	T1CONbits.T1CKPS1		// Timer1 Input Clock Prescale Select bits
#define _T1CKPS0	T1CONbits.T1CKPS0		// 		10 = 1:4 Prescale value
#define _T1OSCEN	T1CONbits.T1OSCEN:		// Timer1 Oscillator Enable bit(0 = Timer1 oscillator is shut off)
#define _T1SYNC		T1CONbits.T1SYNC		// Timer1 External Clock Input Synchronization Select bit
#define _TMR1CS		T1CONbits.TMR1CS		// Timer1 Clock Source Select bit(0 = Internal clock (FOSC/4))
#define _T1ON		T1CONbits.TMR1ON		// Timer1 On bit

	#define Timer1_InternalClock()		_TMR1CS = 0;	// Internal clock (FOSC/4)
	#define Timer1_PrescaleSet()		{_T1CKPS1 = 1; _T1CKPS0 = 1;}	// 11 = 1:8 Prescale value
	#define Timer1_startBit_ON()		_T1ON = 1;
	#define Timer1_startBit_OFF()		_T1ON = 0;
	#define Timer1_IntEnable()			_T1IE = 1;
	#define Timer1_IntDisable()			_T1IE = 0;
	#define Timer1_EvetFlag_Clear()		_T1IF = 0;		// BitNA

/* T3CON: TIMER3 CONTROL REGISTER */
#define _T3CKPS1	T3CONbits.T3CKPS1		// Timer3 Input Clock Prescale Select bits
#define _T3CKPS0	T3CONbits.T3CKPS0		// 		10 = 1:4 Prescale value
#define _T3OSCEN	T3CONbits.T3OSCEN:		// Timer3 Oscillator Enable bit(0 = Timer3 oscillator is shut off)
#define _T3SYNC		T3CONbits.T3SYNC		// Timer3 External Clock Input Synchronization Select bit
#define _TMR3CS		T3CONbits.TMR3CS		// Timer3 Clock Source Select bit(0 = Internal clock (FOSC/4))
#define _T3ON		T3CONbits.TMR3ON		// Timer3 On bit

	#define Timer3_InternalClock()		_TMR3CS = 0;	// Internal clock (FOSC/4)
	#define Timer3_PrescaleSet()		{_T3CKPS1 = 1; _T3CKPS0 = 0;}	// 10 = 1:4 Prescale value
	#define Timer3_startBit_ON()		_T3ON = 1;
	#define Timer3_startBit_OFF()		_T3ON = 0;
	#define Timer3_IntEnable()			_T3IE = 1;
	#define Timer3_IntDisable()			_T3IE = 0;
	#define Timer3_EvetFlag_Clear()		_T3IF = 0;		// BitNA

	// **Note** ȉ̊֐͌󒆐gȂB	2010/06/19(y)
		/* 荞݃}XN */
	#define disable_interrupt()
		/* 荞݃}XN */
	#define enable_interrupt()

    /** I/O pin definitions ********************************************/
    #define INPUT_PIN 1
    #define OUTPUT_PIN 0

#endif  //HARDWARE_PROFILE_PIC18F2550_H
