/**********************************************************************
***********************************************************************
*	Processor	: PIC18F2550
*	File Name   : PIC18fTimer.c
*	Summary		: ^C}[PER
*
*	History:
*	Date		Designer	Description
*	07/23/2010	DachsTulip	Created	
*
*	Copyright (c) 2010 by DachsTulip
***********************************************************************
***********************************************************************/
/**
******************************************************************
	C:\Program Files\Microchip\MCC18\h\p18f2550.h
	C:\FootMouseKey\user\mytype.h
			|-> Timer1_startBit_OFF()
	C:\Program Files\Microchip\MCC18\src\pmc_common\Timers\t1open.c

		T1CON: TIMER1 CONTROL REGISTER
			bit 5-4 T1CKPS1:T1CKPS0: Timer1 Input Clock Prescale Select bits
				11 = 1:8 Prescale value
				10 = 1:4 Prescale value
				01 = 1:2 Prescale value
				00 = 1:1 Prescale value
			bit 1 TMR1CS: Timer1 Clock Source Select bit
				0 = Internal clock (FOSC/4)

	QlURL
		http://homepage1.nifty.com/rikiya/software/314.TIMER1.htm
******************************************************************
	*/

/** INCLUDES *******************************************************/
#include "HardwareProfile.h"
#include	"mytype.h"
#include	"PrintSCI.h"


/** VARIABLES ******************************************************/
static	BOOL timer1_ConfFlag = FALSE;
		BOOL timer1_UpFlag = FALSE;
static	BOOL timer3_ConfFlag = FALSE;
		BOOL timer3_UpFlag = FALSE;
static	UINT16 Tm2_Count = 0;

static	UINT16 Timer1_Remain;
static	UINT16 Timer3_Remain;
/** DECLARATIONS ***************************************************/
#pragma code

#if defined(PIC24F_STARTER_KIT)

#elif defined(PIC18F2550)
/**
		FOSC/4 Internal clock
		Fosc=20MHz	:4 Prescale value -> 0.2sec/pulse
					T1CKPS1:T1CKPS0: 11 -> 1.6sec/pulse

**Note** 2010/07/01() PIC18F2550 AbvJE^[݂̂Ȃ̂Œ(_EJ^[@\̂ĂȂB)
	*/

#define		TMR_SCALONG_VALUE	(UINT16)600			// 1.6sec/pulse -> 1msec/625pulse
#define		TMR_REMAIN_VALUE	(UINT16)(100)		// 100msec
void timer_MsecSet(UINT8 tmCHNo, UINT16 msecValue)
{
	UINT16 tmdt = msecValue;
	UINT16 temp;

	if( tmdt >= 10000)	{ tmdt = 10000; }	// max 10sec (10000msec)
	if( tmdt <= 0)		{ tmdt = 1; }		// min 0.1msec

	if(tmdt > TMR_REMAIN_VALUE)
	{
		temp = tmdt;
		tmdt = tmdt % TMR_REMAIN_VALUE;
		temp = temp - tmdt;

		     if(tmCHNo == 1) { Timer1_Remain = temp; }
		else if(tmCHNo == 3) { Timer3_Remain = temp; }
	}
	else
	{
		     if(tmCHNo == 1) { Timer1_Remain = 0; }
		else if(tmCHNo == 3) { Timer3_Remain = 0; }
	}

	tmdt = (tmdt * TMR_SCALONG_VALUE);
	// *** Up counter only ***
	tmdt = tmdt*(-1);
//	PrintSCI("t=%x\r\n", tmdt,0);

	if(tmCHNo == 1)
	{
		TMR1H = (UINT8)(tmdt>>8);
		TMR1L = (UINT8)tmdt;
	}
	else if(tmCHNo == 3)
	{
		TMR3H = (UINT8)(tmdt>>8);
		TMR3L = (UINT8)tmdt;
	}
}

/**
	return	TRUE(1)  : cʂ
			FALSE(0) : cʂȂ
	*/
BOOL timer_RepeatSet(UINT8 tmCHNo)
{
	UINT16 tmdt;
	UINT16 temp;

	     if(tmCHNo == 1) { tmdt = Timer1_Remain; }
	else if(tmCHNo == 3) { tmdt = Timer3_Remain; }

	if(tmdt == 0)
	{
		return(FALSE);
	}
	else if(tmdt > TMR_REMAIN_VALUE)
	{
		temp = tmdt;
		tmdt = tmdt % TMR_REMAIN_VALUE;
		if(tmdt == 0)
		{
			tmdt = TMR_REMAIN_VALUE;
		}
		temp = temp - tmdt;

		     if(tmCHNo == 1) { Timer1_Remain = temp; }
		else if(tmCHNo == 3) { Timer3_Remain = temp; }
	}
	else
	{
		     if(tmCHNo == 1) { Timer1_Remain = 0; }
		else if(tmCHNo == 3) { Timer3_Remain = 0; }
	}

	tmdt = (tmdt * TMR_SCALONG_VALUE);
	// Up counter only
	tmdt = tmdt*(-1);

	if(tmCHNo == 1)
	{
		TMR1H = (UINT8)(tmdt>>8);
		TMR1L = (UINT8)tmdt;
	}
	else if(tmCHNo == 3)
	{
		TMR3H = (UINT8)(tmdt>>8);
		TMR3L = (UINT8)tmdt;
	}
	return(TRUE);
}


void timer_config(UINT8 tmCHNo)
{
	if(tmCHNo == 1)
	{
		Timer1_InternalClock();				// Internal clock (FOSC/4)
		Timer1_PrescaleSet();				// 11 = 1:8 Prescale value
		Timer1_startBit_OFF();
		Timer1_IntDisable();
		Timer1_EvetFlag_Clear();
		timer1_ConfFlag = TRUE;
	}
	else if(tmCHNo == 3)
	{
		Timer3_InternalClock();				// Internal clock (FOSC/4)
		Timer3_PrescaleSet();				// 11 = 1:8 Prescale value
		Timer3_startBit_OFF();
		Timer3_IntDisable();
		Timer3_EvetFlag_Clear();
		timer3_ConfFlag = TRUE;
	}
}

/**
******************************************************************
* Timer1
******************************************************************
	*/
void msecTimer1_Start(UINT16 tmValue)
{
	if( timer1_ConfFlag != TRUE)	// Ă邩H
	{
		return;
	}
	else if( _T1IE == 1)			// ݋H
	{
		return;
	}
	else
	{
		timer_MsecSet(1, tmValue);
		Timer1_startBit_ON();
		Timer1_IntEnable();
		Timer1_EvetFlag_Clear();
		timer1_UpFlag = FALSE;
	}
}

void msecTimer1_Stop(void)
{
	if( timer1_ConfFlag != TRUE)	// Ă邩H
	{
		return;
	}
	Timer1_IntDisable();
	Timer1_startBit_OFF()
}


/**
******************************************************************
* Timer3
******************************************************************
	*/
void msecTimer3_Start(UINT16 tmValue)
{
	if( timer3_ConfFlag != TRUE)	// Ă邩H
	{
		return;
	}
	else if( _T3IE == 1)			// ݋H
	{
		return;
	}
	else
	{
		timer_MsecSet(3, tmValue);
		Timer3_startBit_ON();
		Timer3_IntEnable();
		Timer3_EvetFlag_Clear();
	}
}

void msecTimer3_Stop(void)
{
	if( timer3_ConfFlag != TRUE)	// Ă邩H
	{
		return;
	}
	Timer3_IntDisable();
	Timer3_startBit_OFF()
}


#else
    #error No hardware board defined, see "HardwareProfile.h" and __FILE__
#endif


