/**
****************************************************************
****************************************************************
*	Processor	: PIC18F2550
*	File Name   : PatternData.h
*	
*	History:
*	Date		Designer	Description
*	07/23/2010	DachsTulip	Created	
*
*	Copyright (c) 2010 by DachsTulip
****************************************************************
	*/

#ifndef __PatternData__
#define __PatternData__

#include	"mouseKey.h"

/**
****************************************************************************
 * :        
	C:\Program Files\Microchip\MCC18\h\pconfig.h
			|-> #define I2C_V1

	C:\Program Files\Microchip\MCC18\h\i2c.h
		|-> #define StartI2C()  SSPCON2bits.SEN=1;while(SSPCON2bits.SEN)
	
	C:\Program Files\Microchip\MCC18\src\pmc_common\i2c\i2c_open.c
			  SSPCON1 |= sync_mode;    // select serial mode 
			  			MSSP CONTROL REGISTER 1 (SPI MODE)
			  SSPCON2
			  			MSSP CONTROL REGISTER 2 (I2C MASTER MODE)
			  SSPSTAT |= slew;         // slew rate on/off 
						MSSP STATUS REGISTER (I2C MODE)
		http://www.picfun.com/pic18/mod18x12.html
		SSPCON2WX^I2C[hł̂ݎg
		
		void StartI2C( void )
				SSPCON2bits.SEN = 1;            // initiate bus start condition
		void RestartI2C( void )
				SSPCON2bits.RSEN = 1;           // initiate bus restart condition
		C:\Program Files\Microchip\MCC18\h\i2c.h
			#define StopI2C()  SSPCON2bits.PEN=1;while(SSPCON2bits.PEN)
		void CloseI2C( void )
 ****************************************************************************
	*/

/**
	SSPCON1
		bit 5 SSPEN: Master Synchronous Serial Port Enable bit
		bit 3-0 SSPM3:SSPM0: Master Synchronous Serial Port Mode Select bits
			1000 = I2C Master mode, clock = FOSC/(4 * (SSPADD + 1))(2,3)
	SSPCON2
		bit 6 ACKSTAT: Acknowledge Status bit (Master Transmit mode only)
			0 = Acknowledge was received from slave
		bit 4 ACKEN: Acknowledge Sequence Enable bit(2)
			1 = Initiate Acknowledge sequence on SDA and SCL pins and transmit ACKDT data bit. Automatically
				cleared by hardware.
			0 = Acknowledge sequence Idle
		bit 3 RCEN: Receive Enable bit (Master Receive mode only)(2)
			1 = Enables Receive mode for I2C
		bit 2 PEN: Stop Condition Enable bit(2)
			1 = Initiate Stop condition on SDA and SCL pins. Automatically cleared by hardware.
		bit 0 SEN: Start Condition Enable/Stretch Enable bit(2)
			1 = Initiate Start condition on SDA and SCL pins. Automatically cleared by hardware.
	SSPSTAT
		bit 2 R/W: Read/Write Information bit (Used in I2C mode only.)

	*/


	// 20MHz/16=1.25MHz	CKP: Clock Polarity Select bit(1 = Idle state for clock is a high level)
#define EEPROM_SSPCON1				0x28
#define EEPROM_SSPSTAT				0x00
#define EEPROM_PAGE_MAX				32
#define EEPROM_ADDRESS_MIN			1024		// EEPROMy[Wmax=32Ŋ鐔l
#define EEPROM_ADDRESS_MAX			(8128-1)	// EEPROM=24LC64 8191Byte(64Kb devices)(1FFFh)Ώ
											// 8100/8Byte=1012 data ()
											// 7167/16Byte=447 data () R[heLXgPs̃f[^
/**
	1KByte	XCb`Ǘ(SAT:switch allocation table)	4ByteP
			  2Byte : p[^f[^̐擪AhX
			  2Byte : \
			v4Byte
	*/
#define EEPROM_ADDRESS_INFO_INTERVAL	4
#define HEX_ROTARY_DIP_SW				16
#define EEPROM_FOOTSW_INTERVAL			(EEPROM_ADDRESS_INFO_INTERVAL * HEX_ROTARY_DIP_SW)

#define EEPROM_DATA_LENGTH				16
#define HIDCODE_BUFFER_ARRAY			10
#define EEPROM_WRITE_CONTROL_CODE		0xA0			// 24LC64 Control Code
#define EEPROM_READ_CONTROL_CODE		(EEPROM_WRITE_CONTROL_CODE | 0x01)
#define EEPROM_PAGE_ADDRESS_MASK		0x001F


/**
***********************************************************************
* Function Declaration
************************************************************************
*/
extern void ClearReadIndex(void);
extern void ClearWriteIndex(void);

// **Note** ȉGetPatternData()`ȂƁAreturnlςȒlƂȂB	2010/06/07()
//		(??retErr=65h <- result=0)
extern Emulate_Error_t GetHidCodeDataFromEEPRom( GetPattData_t, FootSwitch_t, UINT8 *);
extern Emulate_Error_t SetHidCodeDataToEEPRom(UINT8 *);


#endif		//  __PatternData__
