/********************************************************************
*********************************************************************
*	Processor		: PIC18F2550
*	FileName		: mouseKey.c
*	Printed board	: YM-001A
*	Complier		: MPLAB C Compiler for PIC18 v3.35 (for PIC18F2550)
*	USB Framework	Microchip Application Libraries v2009-08-31.EXE
*		( for PIC18)
*	Reference source: Keyboard.c or mouse.c etc.
*
*	History:
*	Date		Designer	Description
*	08/11/2010	DachsTulip	Created
*	10/28/2010	"#endif" of "#if defined(__18CXX)" had come off.
*
*	Copyright (c) 2010 by DachsTulip
********************************************************************
********************************************************************/
/**
********************************************************************
	PIC18F2550 dl
		vOTCY	F32KoCg
		RAMTCY				F2048oCg
		I/O					F24
		pbP[W				F28sDIP
		x					F48MHz
		EEPROM					F256oCg
		f[^[Ro[^		FA/D 10ch x 10bit
		C^[tF[X		F 1-A/E/USART, 1-MSSP(SPI/I2C), USB
********************************************************************
	*/

#ifndef USBMOUSE_KEY_C
#define USBMOUSE_KEY_C

/** INCLUDES *******************************************************/
#include "Compiler.h"
#include "usb_config.h"
#include "./USB/usb_device.h"
//				|-> extern volatile unsigned char hid_report_out[HID_INT_OUT_EP_SIZE];
//				|-> extern volatile unsigned char hid_report_in[HID_INT_IN_EP_SIZE];

#include "./USB/usb.h"
#include "HardwareProfile.h"
#include "./USB/usb_function_hid.h"

#include	"mytype.h"
#include	"mouseKey.h"
#include	"PatternData.h"
#include	"PIC18fTimer.h"
#include	"PrintSCI.h"
//				|-> #include	"picSio.h"


/**
	C:\Program Files\Microchip\MCC18\h\p18f2550.h
					|-> ADCON1
	*/

/**
***********************************************************************
* debug Definitions
************************************************************************
	*/
 #define _ERROR_PRINTF_
#if defined(_ERROR_PRINTF_)
	#if defined(__SIO_ENABLE__)
		#define ERROR_PRINTF(__dbg) __dbg
	#else
		#define ERROR_PRINTF(__dbg)
	#endif
#else
	#define ERROR_PRINTF(__dbg)
#endif


// #define _DEBUG_PRINTF_
#if defined(_DEBUG_PRINTF_)
	#if defined(__SIO_ENABLE__)
		#define DEBUG_PRINTF(__dbg) __dbg
	#else
		#define DEBUG_PRINTF(__dbg)
	#endif
#else
	#define DEBUG_PRINTF(__dbg)
#endif

// #define _TEST_PRINTF_
#if defined(_TEST_PRINTF_)
	#if defined(__SIO_ENABLE__)
		#define TEST_PRINTF(__dbg) __dbg
	#else
		#define TEST_PRINTF(__dbg)
	#endif
#else
	#define TEST_PRINTF(__dbg)
#endif


// #define _DEBUG_LED_
#if defined(_DEBUG_LED_)
	#define DEBUG_LED(__dbg) __dbg
	#define MONITOR_LED(__mon)
#else
	#define DEBUG_LED(__dbg)
	#define MONITOR_LED(__mon) __mon
#endif


/** CONFIGURATION **************************************************/
/**
	C:\Program Files\Microchip\MCC18\mpasm\P18F2550.INC
	*/
#if defined(PIC18F2550)
						// CONFIG1H: CONFIGURATION REGISTER 1 HIGH (BYTE ADDRESS 300001h)

//        #pragma config FOSC = HSPLL_HS	// VXeNbN48MH
											// 111x = HS oscillator, PLL enabled (HSPLL)
		#pragma config FOSC = HS			// VXeNbN20MH
											// 110x = HS oscillator (HS)
											// LݒłȂƁASIÕNbNݒ肪ɏoȂB
        #pragma config WDT = OFF			// HW Disabled - SW Controlled
		#pragma config PLLDIV = 5			// (20 MHz crystal on PICDEM FS USB board)
											// 96MHz PLL Prescalar(1A2A3A4A5A6A10A12̂݁j:Og4MH
											// 20MHz  5 = 4MHz  96MHi=4MHz~24 :Œj
		#pragma config CPUDIV = OSC1_PLL2	// CPU System Clock Postscaler (96 MHz PLL Src: /2)

		#pragma config USBDIV = 2			// Clock source from 96MHz PLL/2
        #pragma config PWRT = ON			// p[Abv^C}ON
        #pragma config BOR = ON				// uEAEgZbgONin[hEFÂ݁j
        #pragma config BORV = 3				// Brown out Voltage BORV=3 2.05v (Minimum setting)
											// refF BORV=0  4.59v   BORV=1  4.33v   BORV=2  2.79v
        #pragma config LVP = OFF			// dOFF (Single-Supply ICSP disabled)
		#pragma config VREGEN = ON			// USB Voltage Regulator
        #pragma config MCLRE = ON			// MCLR pin enabled; RE3 input pin disabled
        #pragma config PBADEN = OFF			// ZbgŃ|[gB̑SAiO|[gfW^I/OɃZbg
											// PORTB A/D Enable bit = 0 (CONFIG3H)

// 2010/06/22() ȉ̐ݒ薢ΉiKv̍l@܂jHHHHHHHHHH
//#pragma config PBADEN   = ON // PORTB<4:0> pins are configured as analog input channels on Reset 
//#pragma config LPT1OSC  = OFF // Timer1 configured for higher power operation
//#pragma config CCP2MX   = ON // CCP2 input/output is multiplexed with RB3
//#pragma config STVREN   = ON // Stack full/underflow will cause Reset
//#pragma config DEBUG    = OFF // Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins 


#else
    #error No hardware board defined, see "HardwareProfile.h" and __FILE__
#endif


/** ENUM ******************************************************/

typedef enum
{
	 EMULATE_IDLE
	,EMULATE_KEYBOARD
	,EMULATE_MOUSE
	,EMULATE_TIMER
	,EMULATE_NON

} EmulateMode_t;

typedef enum
{
	 SEQUENCE_IDLE
	,SEQUENCE_START
	,SEQUENCE_NEXT

} SequenceInfo_t;


typedef enum
{
	 DEVICE_KEYBOARD
	,DEVICE_MOUSE
	,DEVICE_NON

} DeviceSelect_t;

enum
{
	 REPORT_ID_NON = (0)
	,REPORT_ID_IO
	,REPORT_ID_KEY
	,REPORT_ID_MOUSE
};


/** VARIABLES ******************************************************/
#pragma udata
BYTE			old_FSW1,old_FSW2,old_FSW3,old_FSW4;
BYTE			old_SW1;

// mouse
char			buffer_m[5];
// key
char			buffer_k[8];
USB_HANDLE		lastINTransmission;
USB_HANDLE		USBOutHandle;

BOOL			BlinkStatusValid;
DWORD			CountdownTimerToShowUSBStatusOnLEDs;

DeviceSelect_t  NextDevSelect;


/**
	C:\FootMouseKey\user\PatternData.h
			|-> #define HIDCODE_BUFFER_ARRAY			10
	*/
BYTE	GetCodeBuffer[HIDCODE_BUFFER_ARRAY];

/**
***********************************************************************
* Definitions
************************************************************************
	*/
/**
	C:\FootMouseKey\system\Usb\usb_device.c
		#if defined(USB_USE_HID)
    		volatile unsigned char hid_report_out[HID_INT_OUT_EP_SIZE];
    		volatile unsigned char hid_report_in[HID_INT_IN_EP_SIZE];
		#endif
	C:\FootMouseKey\system\Include\Usb\usb_device.h
		#if defined(USB_USE_HID)
	    //class specific data buffers
		extern volatile unsigned char hid_report_out[HID_INT_OUT_EP_SIZE];
		extern volatile unsigned char hid_report_in[HID_INT_IN_EP_SIZE];
	#endif
	*/
// ** data area ******************
#define hid_report_mus			hid_report_in
#define hid_report_key			hid_report_in
#define hid_report_send			hid_report_in

#define GeneralInOutBuffer		hid_report_out

 
// ** Foot Switch ******************
#define ENABLE_FSW_1_DETECT
#define ENABLE_FSW_2_DETECT
#define ENABLE_FSW_3_DETECT
#define ENABLE_FSW_4_DETECT

// I"0"f[^p\Rɏグ鏈Lɂ邩̃RpCXCb`
// tbgXCb`͂ȂĂAp\R"k,0,0,0""m,0,0,0" ݂ɏグĂB
// "k,0,0,0"(key up)́AeLXgR[hf[^ŋL鎖ƂB		2010/07/01()
// #define COMPULSORY_ZERO_UPFOR_PC


/** PRIVATE PROTOTYPES *********************************************/
SwitchReturn_t Switch1IsPressed(void);
SwitchReturn_t FootSwIsPressed(FootSwitch_t *footNo);

void			BlinkUSBStatus(void);
void			Emulate_Mouse(void);
static void 	InitializeSystem(void);
void			ProcessIO(void);
void			YourHighPriorityISRCode();
void			YourLowPriorityISRCode();

void			USBHIDCBSetReportComplete(void);

void			Emulate_MouseKey(void);

void			InitIntervalTimers(void);
void			GeneralInit(void);
void			ChattRemovalInit(void);				// 2010/08/11()
Emulate_Error_t		ReciveKeyMouseTextCode(void);
Emulate_Error_t 	SendPalams(void);

void			pic18fT1Interrupt(void);
void			pic18fT3Interrupt(void);

// gݗČ㓮eXg
	CIRCUIT_TEST(void cirCuitTest();)

/** VECTOR REMAPPING ***********************************************/
#if defined(__18CXX)
	//On PIC18 devices, addresses 0x00, 0x08, and 0x18 are used for
	//the reset, high priority interrupt, and low priority interrupt
	//vectors.  However, the current Microchip USB bootloader 
	//examples are intended to occupy addresses 0x00-0x7FF or
	//0x00-0xFFF depending on which bootloader is used.  Therefore,
	//the bootloader code remaps these vectors to new locations
	//as indicated below.  This remapping is only necessary if you
	//wish to program the hex file generated from this project with
	//the USB bootloader.  If no bootloader is used, edit the
	//usb_config.h file and comment out the following defines:
	//#define PROGRAMMABLE_WITH_USB_HID_BOOTLOADER
	//#define PROGRAMMABLE_WITH_USB_LEGACY_CUSTOM_CLASS_BOOTLOADER
	
	#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)
		#define REMAPPED_RESET_VECTOR_ADDRESS			0x1000
		#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x1008
		#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x1018
	#elif defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)	
		#define REMAPPED_RESET_VECTOR_ADDRESS			0x800
		#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x808
		#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x818
	#else	
		#define REMAPPED_RESET_VECTOR_ADDRESS			0x00
		#define REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS	0x08
		#define REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS	0x18
	#endif
	
	#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
	extern void _startup (void);        // See c018i.c in your C18 compiler dir
	#pragma code REMAPPED_RESET_VECTOR = REMAPPED_RESET_VECTOR_ADDRESS
	void _reset (void)
	{
	    _asm goto _startup _endasm
	}
	#endif
	#pragma code REMAPPED_HIGH_INTERRUPT_VECTOR = REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS
	void Remapped_High_ISR (void)
	{
	     _asm goto YourHighPriorityISRCode _endasm
	}
	#pragma code REMAPPED_LOW_INTERRUPT_VECTOR = REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS
	void Remapped_Low_ISR (void)
	{
	     _asm goto YourLowPriorityISRCode _endasm
	}
	
	#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_MCHPUSB_BOOTLOADER)
	//Note: If this project is built while one of the bootloaders has
	//been defined, but then the output hex file is not programmed with
	//the bootloader, addresses 0x08 and 0x18 would end up programmed with 0xFFFF.
	//As a result, if an actual interrupt was enabled and occured, the PC would jump
	//to 0x08 (or 0x18) and would begin executing "0xFFFF" (unprogrammed space).  This
	//executes as nop instructions, but the PC would eventually reach the REMAPPED_RESET_VECTOR_ADDRESS
	//(0x1000 or 0x800, depending upon bootloader), and would execute the "goto _startup".  This
	//would effective reset the application.
	
	//To fix this situation, we should always deliberately place a 
	//"goto REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS" at address 0x08, and a
	//"goto REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS" at address 0x18.  When the output
	//hex file of this project is programmed with the bootloader, these sections do not
	//get bootloaded (as they overlap the bootloader space).  If the output hex file is not
	//programmed using the bootloader, then the below goto instructions do get programmed,
	//and the hex file still works like normal.  The below section is only required to fix this
	//scenario.
/**
		C:\FootMouseKey\user\picSio.h
			|-> #define __SIO_ENABLE__
	*/

	#pragma code HIGH_INTERRUPT_VECTOR = 0x08
	void High_ISR (void)
	{
	     _asm goto REMAPPED_HIGH_INTERRUPT_VECTOR_ADDRESS _endasm
	}
	#pragma code LOW_INTERRUPT_VECTOR = 0x18
	void Low_ISR (void)
	{
	     _asm goto REMAPPED_LOW_INTERRUPT_VECTOR_ADDRESS _endasm
	}
	#endif	//end of "#if defined(PROGRAMMABLE_WITH_USB_HID_BOOTLOADER)||defined(PROGRAMMABLE_WITH_USB_LEGACY_CUSTOM_CLASS_BOOTLOADER)"

	#pragma code

	//These are your actual interrupt handling routines.
	#pragma interrupt YourHighPriorityISRCode
	void YourHighPriorityISRCode()
	{
		//Check which interrupt flag caused the interrupt.
		//Service the interrupt
		//Clear the interrupt flag
		//Etc.

        #if defined(USB_INTERRUPT)
			/**
				C:\FootMouseKey\system\Usb\usb_device.c
				*/
	        USBDeviceTasks();
        #endif
		if (PIR1bits.TMR1IF)				// TMR1 Overflow Interrupt Flag bit
		{
			pic18fT1Interrupt();
		}
		// ȉelse if ifɕύXBłȂƐ擪ςȂƑ̊ݏɉȂB
		if (PIR2bits.TMR3IF)				// TMR3 Overflow Interrupt Flag bit
		{
			pic18fT3Interrupt();
		}

		#if defined(__SIO_ENABLE__)			// VAʐM\[XLɂ`
		if (PIR1bits.RCIF)
		{
//			DEBUG_LED(mLED_3_Toggle();)
			EUSART_RXInterrupt();
		}
		// 2010/06/21()
		// TXIF̏ƁAEUSART_TXInterrupt()֐Ă܂̂TXIȄǉ
		if ((PIR1bits.TXIF) && (PIE1bits.TXIE))
		{
			EUSART_TXInterrupt();
		}
		#endif								// #if defined(__SIO_ENABLE__)
	}	//This return will be a "retfie fast", since this is in a #pragma interrupt section 
	#pragma interruptlow YourLowPriorityISRCode
	void YourLowPriorityISRCode()
	{
		//Check which interrupt flag caused the interrupt.
		//Service the interrupt
		//Clear the interrupt flag
		//Etc.
	
	}	//This return will be a "retfie", since this is in a #pragma interruptlow section 

#endif				// "__18CXX" 2010/10/28()


/** VECTOR REMAPPING ***********************************************/
#if defined(__18CXX)
    #if defined(__18F2550)
		#define		USER_SW		PORTCbits.RC2
		// 
	    #if defined(ROTARY_DIP_SW_REALCODE)		// AER[h
			#define		DR_SC16()	(~(((PORTA & 0x30)>>4) | ((PORTC & 0x03)<<2))) & 0x0f
		#else									// Rv^ER[h
			#define		DR_SC16()	(((PORTA & 0x30)>>4) | ((PORTC & 0x03)<<2)) & 0x0f;
		#endif
	#endif

#endif				// "__18CXX" 2010/10/28()


BYTE	DigSW;
BYTE	mouseSW;


/** DECLARATIONS ***************************************************/
#pragma code

/**
	adde 2010/05/06() 18:05 \[XϓLϐ
		002DE0: 0012 4D00 7961 3020 2036 3032 3031 3120 Mya0  602011 
                                           2010     
		002DF0: 3A38 3530 313A 0036 6B00 0000 6B58 0004 :8501:6kkX
                18:   05   :16
	*/
const char COMP_DATE[] = __DATE__ " " __TIME__;


/**
****************************************************************************
* HIDR[hMV[PX
****************************************************************************
	*/
SequenceInfo_t SequenceFlag = SEQUENCE_IDLE;

void StartSequence(void)  { SequenceFlag = SEQUENCE_START;}
void NextSequence(void) { SequenceFlag = SEQUENCE_NEXT;}
void StopSequence(void)   { SequenceFlag = SEQUENCE_IDLE;}

BOOL SequenceRunning(void)
{
	if(SequenceFlag != SEQUENCE_IDLE)	return TRUE;
	else								return FALSE;
}


/********************************************************************
 * Function:        void main(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        Main program entry point.
 * Note:            None
 *******************************************************************/
#if defined(__18CXX)
void main(void)
#else
int main(void)
#endif
{
    InitializeSystem();

    #if defined(USB_INTERRUPT)
        USBDeviceAttach();
    #endif

    while(1)
    {
		/**
			C:\FootMouseKey\user\usb_config.h
					|-> //#define USB_POLLING		|[OA݂ŏĂB
			*/
		#if defined(USB_POLLING)
		// Check bus status and service USB interrupts.
			USBDeviceTasks(); // Interrupt or polling method.
        #endif

		// Application-specific tasks.
		// Application related code may be added here, or in the ProcessIO() function.
        ProcessIO();        
    }//end while
}//end main


/******************************************************************************
 * Function:        void UserInit(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        This routine should take care of all of the demo code
 *                  initialization that is required.
 * Note:            
 *****************************************************************************/
void UserInit(void)
{
    //Initialize all of the LED pins
    mInitAllLEDs();
    //Initialize all of the DIG SW pins
	mInitAllDGSW();

    //Initialize all of the push buttons
    mInitAllSwitches();
    old_FSW1 = FSW_1;
    old_FSW2 = FSW_2;
    old_FSW3 = FSW_3;
    old_FSW4 = FSW_4;
    old_SW1 = SW1;

    //Initialize all of the mouse data to 0,0,0 (no movement)
    buffer_m[0]=buffer_m[1]=buffer_m[2]=0;

    //initialize the variable holding the handle for the last
    // transmission
	lastINTransmission = 0;
    USBOutHandle = 0;

	// 2009/10/24 added rev.io
	// ^C}[PERLɂB
	timer_config(1);
	timer_config(3);

#if defined(__SIO_ENABLE__)			// VAʐM\[XLɂ`
	sci_initial();
#endif								// #if defined(__SIO_ENABLE__)

	GeneralInit();					// ėpR}hf[^͏
	InitIntervalTimers();
	ChattRemovalInit();				// 2010/08/11() `^OXCb` 

	NextDevSelect = DEVICE_KEYBOARD;
	StopSequence();

	MONITOR_LED(mLED_3_Off();)
	MONITOR_LED(mLED_4_Off();)

}//end UserInit

/********************************************************************
 * Function:        static void InitializeSystem(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        InitializeSystem is a centralize initialization
 *                  routine. All required USB initialization routines
 *                  are called from here.
 *                  User application initialization routine should
 *                  also be called from here.                  
 * Note:            None
 *******************************************************************/
static void InitializeSystem(void)
{
    #if (defined(__18CXX) & !defined(PIC18F87J50_PIM))
        // A/D CONTROL REGISTER 1
        // PCFG3:PCFG0: A/D Port Configuration Control bits:
        ADCON1 |= 0x0F;            // Default all pins to digital
    #elif defined(__C30__)
        AD1PCFGL = 0xFFFF;
    #elif defined(__C32__)
        AD1PCFG = 0xFFFF;
    #endif

   #if defined(PIC24FJ64GB004_PIM)
	//On the PIC24FJ64GB004 Family of USB microcontrollers, the PLL will not power up and be enabled
	//by default, even if a PLL enabled oscillator configuration is selected (such as HS+PLL).
	//This allows the device to power up at a lower initial operating frequency, which can be
	//advantageous when powered from a source which is not gauranteed to be adequate for 32MHz
	//operation.  On these devices, user firmware needs to manually set the CLKDIV<PLLEN> bit to
	//power up the PLL.
    {
        unsigned int pll_startup_counter = 600;
        CLKDIVbits.PLLEN = 1;
        while(pll_startup_counter--);
    }

    //Device switches over automatically to PLL output after PLL is locked and ready.
    #endif

    #if defined(USE_USB_BUS_SENSE_IO)
        tris_usb_bus_sense = INPUT_PIN; // See HardwareProfile.h
    #endif
    
    #if defined(USE_SELF_POWER_SENSE_IO)
        tris_self_power = INPUT_PIN;	// See HardwareProfile.h
    #endif
    
    UserInit();

/**
		C:\FootMouseKey\system\Usb\usb_device.c
			|-> void USBDeviceInit(void)
			C:\FootMouseKey\system\Include\Usb\usb_device.h
				|-> USBClearInterruptRegister(U1IR);
					|-> #define USBClearInterruptRegister(reg) reg = 0;
			C:\Microchip Solutions\Microchip\Include\Usb\usb_hal_pic18.h
				|-> #define U1IR UIR

		C:\Program Files\Microchip\MCC18\h\p18cxxx.h
				|-> #include <p18f2550.h>
			C:\Program Files\Microchip\MCC18\h\p18f2550.h
				extern volatile near unsigned char       UIR;
				extern volatile near struct {
				  unsigned URSTIF:1;
				  unsigned UERRIF:1;
				  unsigned ACTVIF:1;
				  unsigned TRNIF:1;
				  unsigned IDLEIF:1;
				  unsigned STALLIF:1;
				  unsigned SOFIF:1;
				} UIRbits;
	*/
    USBDeviceInit();	//usb_device.c.  Initializes USB module SFRs and firmware
    					//variables to known states.
}//end InitializeSystem


/********************************************************************
 * Function:        void ProcessIO(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        This function is a place holder for other user
 *                  routines. It is a mixture of both USB and
 *                  non-USB tasks.
 * Note:            None
 *******************************************************************/
void ProcessIO(void)
{   
    //Blink the LEDs according to the USB device status
    BlinkUSBStatus();

//	DEBUG_LED(mLED_4_Toggle();)

    // User Application USB tasks
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;
//			|-> DETACHED_STATE(0x00)
//			|-> ATTACHED_STATE(0x01)
//			|-> POWERED_STATE(0x02)
//			|-> DEFAULT_STATE(0x04)
//			|-> ADR_PENDING_STATE(0x08)
//			|-> ADDRESS_STATE(0x10)
//			|-> CONFIGURED_STATE(0x20)

//	DEBUG_LED(mLED_3_Toggle();)

		// [^fbvXCb`
	#if defined(PIC24F_STARTER_KIT)||defined(PIC18F2550)
		DigSW = DR_SC16();
	#endif

// gݗČ㓮eXg
	CIRCUIT_TEST(cirCuitTest();)

	ReciveKeyMouseTextCode();
    Emulate_MouseKey();

}//end ProcessIO


/******************************************************************************
 * Keyboard
 *****************************************************************************/
/**
		擪1byteɂ̓VtgL[Rg[L[̏Ԃ܂B
		bit7:EGUIA܂vpeBL[(RIGHT GUI)
		bit6:EAlt						(RIGHT ALT)
		bit5:EVtg					(RIGHT SHIFT)
		bit4:ERg[				(RIGHT CTRL)
		bit3:GUIA܂WindowsL[	(LEFT GUI)
		bit2:Alt						(LEFT ALT)
		bit1:Vtg					(LEFT SHIFT)
		bit0:Rg[				(LEFT CTRL)

		Usage Minimum (0x00) Assigns first of 101-key usages.
		Usage Maximum (0x65) Assigns last of 101-key usages.

		Key codes \
		D:\MANUAL\70_֌W\HID\USBL[{[h̃L[R[h.mht
	*/

/******************************************************************************
 * Mouse
 *****************************************************************************/


/**
****************************************************************************
* ̃L[E}EX[h擾
* R}hs擪́A'k': L[R[h	'm': }EXR[h	't': ^C}R[h
****************************************************************************
	*/
EmulateMode_t GetNextSequenceMode(void)
{
	UINT8 device = GetCodeBuffer[0];

	     if((device == 'k') || (device == 'K'))		return(EMULATE_KEYBOARD);
	else if((device == 'm') || (device == 'M'))		return(EMULATE_MOUSE);
	else if((device == 't') || (device == 'T'))		return(EMULATE_TIMER);
	else											return(EMULATE_NON);
}

/**
****************************************************************************
* C^[o^C}[ݒ
****************************************************************************
	*/
UINT16 IntervalTimers;
void InitIntervalTimers(void)
{
	IntervalTimers = TM10MSEC;
}

Emulate_Error_t SetIntervalTimers(void)
{
	IntervalTimers = (UINT16)GetCodeBuffer[1];
	// minimum Interval Timers =1msec
	if(IntervalTimers == 0)		IntervalTimers=1;
	TEST_PRINTF(PrintSCI("IntvalTm=%x\r\n", tms, 0);)
}

/**
****************************************************************************
* 
****************************************************************************
	*/
Emulate_Error_t SequenceStep(EmulateMode_t mode)
{
	if(( mode == EMULATE_KEYBOARD) || ( mode == EMULATE_MOUSE))
	{
		// OK goto next job
	}
	else
	{
		return(EMU_CALL_FAILED);
	}

	if(GetCodeBuffer[1] == 0xFF)
	{
		return(EMU_CALL_COMPLET);
	}
	else
	{
		if( mode == EMULATE_KEYBOARD)
		{
			buffer_k[0] = GetCodeBuffer[1];
			buffer_k[1] = GetCodeBuffer[2];
		   	buffer_k[2] = GetCodeBuffer[3];
		   	buffer_k[3] = GetCodeBuffer[4];
		   	buffer_k[4] = GetCodeBuffer[5];
		   	buffer_k[5] = GetCodeBuffer[6];
		   	buffer_k[6] = GetCodeBuffer[7];
		   	buffer_k[7] = GetCodeBuffer[8];
		}
		else if( mode == EMULATE_MOUSE)
		{
			buffer_m[0] = GetCodeBuffer[1];
			buffer_m[1] = GetCodeBuffer[2];
		   	buffer_m[2] = GetCodeBuffer[3];
		}
		return(EMU_CALL_OK);
	}
}

/**
****************************************************************************
* }EXR[h
****************************************************************************
*/
Emulate_Error_t emulateMouse(EmulateMode_t mode)
{   
	Emulate_Error_t		retErr;

	if( mode == EMULATE_MOUSE)
	{
		retErr = SequenceStep(EMULATE_MOUSE);
//			|-> EMU_CALL_OK
//			|-> EMU_CALL_FAILED
//			|-> EMU_CALL_COMPLET
		if(retErr != EMU_CALL_OK)
		{
			return(retErr);
		}
	}

	if(!HIDTxHandleBusy(lastINTransmission))			// MfBmF handle->STAT.UOWN
	{
		hid_report_mus[0] = REPORT_ID_MOUSE;

		if( mode == EMULATE_MOUSE)
		{
			hid_report_mus[1] = buffer_m[0];
			hid_report_mus[2] = buffer_m[1];
			hid_report_mus[3] = buffer_m[2];
		}
		else
		{
			hid_report_mus[1] = 0;
			hid_report_mus[2] = 0;				// X-Vector
			hid_report_mus[3] = 0;				// Y-Vector
		}

		/**
				C:\FootMouseKey\user\usb_config.h
					|-> #define HID_EP 	1
			*/
		//Send the 3 byte packet over USB to the host.
		lastINTransmission = HIDTxPacket(HID_EP, (BYTE*)hid_report_mus, 0x04);

		return(EMU_CALL_OK);
	}
	return(EMU_CALL_FAILED);
}

/**
****************************************************************************
* L[R[h
****************************************************************************
*/
Emulate_Error_t emulateKeyboard(EmulateMode_t mode)
{
	Emulate_Error_t		retErr;

	if( mode == EMULATE_KEYBOARD)
	{
		retErr = SequenceStep(EMULATE_KEYBOARD);
//			|-> EMU_CALL_OK
//			|-> EMU_CALL_FAILED
//			|-> EMU_CALL_COMPLET
		if(retErr != EMU_CALL_OK)
		{
//			ERROR_PRINTF(PrintSCI("retErr=%x\r\n", (UINT16)retErr,0);)
			return(retErr);
		}
	}

	retErr = EMU_CALL_FAILED;
	if(!HIDTxHandleBusy(lastINTransmission))			// MfBmF handle->STAT.UOWN
	{
		hid_report_key[0]= REPORT_ID_KEY;

		if( mode == EMULATE_KEYBOARD)
		{
			hid_report_key[1]= buffer_k[0];
			hid_report_key[2]= buffer_k[1];
			hid_report_key[3]= buffer_k[2];
			hid_report_key[4]= buffer_k[3];
			hid_report_key[5]= buffer_k[4];
			hid_report_key[6]= buffer_k[5];
			hid_report_key[7]= buffer_k[6];
			hid_report_key[8]= buffer_k[7];
		}
		else
		{
			//Loadthe HID buffer
			hid_report_key[1]= 0;
			hid_report_key[2]= 0;
			hid_report_key[3]= 0;
			hid_report_key[4]= 0;
			hid_report_key[5]= 0;
			hid_report_key[6]= 0;
			hid_report_key[7]= 0;
			hid_report_key[8]= 0;
		}
		//Send the 8 byte packet over USB to the host.
		lastINTransmission= HIDTxPacket(HID_EP, (BYTE*)hid_report_key, 0x09);
		retErr = EMU_CALL_OK;
	}

	return(retErr);
}

/**
****************************************************************************
* }EXEL[R[hC
****************************************************************************
	*/
BOOL retryFlag;
FootSwitch_t FootSWNo;
void Emulate_MouseKey(void)
{
	SwitchReturn_t		retSw;
	Emulate_Error_t		retErr;
	FootSwitch_t		footRial;
	EmulateMode_t		eMode;

	retSw = FootSwIsPressed(&footRial);
//	DEBUG_PRINTF(PrintSCI((ROM BYTE *)("retSw=%x\r\n"), (UINT16)retSw,0);)
//	DEBUG_LED(mLED_3_Toggle();)

	if( retSw == SWITCH_EDGE_ON)
	{
		if( SequenceRunning() == TRUE) { return;}

//		DEBUG_LED(PrintSCI((ROM BYTE *)("RCSTA:%x\r\n"), (UINT16)RCSTA, 0);)
//		DEBUG_LED(mLED_4_Toggle();)

		DEBUG_PRINTF(PrintSCI((ROM BYTE *)("SWITCH_EDGE_ON=%x\r\n"), (UINT16)footRial,0);)

		FootSWNo = footRial;
		StartSequence();			// Sequence Running
		retryFlag = FALSE;

		// R[h擪ŁAInterval Timers = 10msec (ftHgl)ƂB
		// EEPROMf[^Ƀ^C}[ݒ肪΂̒lɕύXB
		InitIntervalTimers();
		msecTimer1_Start((UINT16)TM10MSEC);
	}
	else if( retSw == SWITCH_TIME_UP)
	{
		if( SequenceRunning() == FALSE) { return;}

		if(retryFlag == FALSE)
		{
			retErr = EMU_CALL_FAILED;

			switch(SequenceFlag)
			{
				case SEQUENCE_START:
				{
					// EEPROML̃XCb`Ǘe[u(SAT:switch allocation table)
					// L[E}EX̃p^[f[^擾B
					retErr = GetHidCodeDataFromEEPRom( GET_PATTERN_TOP, FootSWNo, &GetCodeBuffer[0]);
					TEST_PRINTF(PrintSCI("retErr=%x\r\n", (UINT16)retErr, 0);)
					NextSequence();
					break;
				}
				case SEQUENCE_NEXT:
				{
					retErr = GetHidCodeDataFromEEPRom( GET_PATTERN_NEXT, FootSWNo, &GetCodeBuffer[0]);
					break;
				}
				default:
				{
					ERROR_PRINTF(PrintSCI((ROM BYTE *)("err SequenceFlag=%x\r\n"), (UINT16)SequenceFlag,0);)
					break;
				}
			}

			if(retErr != EMU_CALL_OK)
			{
				ERROR_PRINTF(PrintSCI((ROM BYTE *)("err GetPattern=%x\r\n"), (UINT16)retErr,0);)
				StopSequence();
				return;
			}
		}

		{
			eMode = GetNextSequenceMode();
			switch(eMode)
			{
				case EMULATE_KEYBOARD:
				{
					retErr = emulateKeyboard( EMULATE_KEYBOARD);
//						|-> EMU_CALL_OK
//						|-> EMU_CALL_FAILED
//						|-> EMU_CALL_COMPLET
					NextDevSelect = DEVICE_KEYBOARD;
					break;
				}
				case EMULATE_MOUSE:
				{
					retErr = emulateMouse( EMULATE_MOUSE);
//						|-> EMU_CALL_OK
//						|-> EMU_CALL_FAILED
//						|-> EMU_CALL_COMPLET
					NextDevSelect = DEVICE_MOUSE;
					break;
				}
				case EMULATE_TIMER:
				{
					SetIntervalTimers();
					break;
				}
				default:
				{
					ERROR_PRINTF(PrintSCI((ROM BYTE *)("err Next Seq Mode\r\n"), 0,0);)
					break;
				}
			}
		}

		if(retErr == EMU_CALL_COMPLET)
		{
			TEST_PRINTF(PrintSCI("complet\r\n", 0,0);)
			StopSequence();
			return;
		}
		else if(retErr == EMU_CALL_OK)
		{
			msecTimer1_Start(IntervalTimers);
			retryFlag = FALSE;
		}
		else
		{
			// MfBmFŎsA10msecăgC
			msecTimer1_Start((UINT16)TM10MSEC);
			retryFlag = TRUE;
		}
	}

#if defined(COMPULSORY_ZERO_UPFOR_PC)
	// ȉpXƁAI"0"f[^p\Rɏグ鏈ƂȂ܂B
	else if(( retSw == SWITCH_ALWAYS_ON) || ( retSw == SWITCH_ALWAYS_OFF))
	{
		// tbgXCb`́Ap\R"0"f[^グȂ		2010/06/08()
		if( SequenceRunning() == FALSE)
		{
			DEBUG_PRINTF(PrintSCI((ROM BYTE *)("sw always off\r\n"), 0,0);)
			if(NextDevSelect == DEVICE_KEYBOARD)
			{
				retErr = emulateKeyboard( EMULATE_NON);
				if(retErr == EMU_CALL_OK)
				{
					NextDevSelect = DEVICE_MOUSE;
				}
			}
			else
			{
				retErr = emulateMouse( EMULATE_NON);
				if(retErr == EMU_CALL_OK)
				{
					NextDevSelect = DEVICE_KEYBOARD;
				}
			}
		}
	}
#endif				// COMPULSORY_ZERO_UPFOR_PC
}


/**
****************************************************************************
* ėpR}hf[^͏
****************************************************************************
	*/
void GeneralInit(void)
{
/**
	C:\FootMouseKey\user\PatternData.c
	EEPROM
	*/
	ClearReadIndex();
	ClearWriteIndex();
}

/**
****************************************************************************
* ėpR}hf[^
	*Note*	IR}h k,hFF,hFF,hFF 鎞ԎMȂ	2010/06/07()
			IɏIR}ht鎖		̂ƂΉ		2010/07/01()
****************************************************************************
	*/
Emulate_Error_t		reciveRetErr;

BOOL	CodeDataFlag = FALSE;
BOOL CheckRetErrOfCodeData(void) { return(CodeDataFlag);}
void SetRetErrOfCodeData(void) { CodeDataFlag = TRUE;}
void ReSetRetErrOfCodeData(void) { CodeDataFlag = FALSE;}

Emulate_Error_t ReciveKeyMouseTextCode(void)
{
	BYTE				BuffNo = DigSW;

	// L[E}EXR[h̎MH
	if(CheckRetErrOfCodeData())
	{
		if(!HIDTxHandleBusy(lastINTransmission))			// MfBmF
		{
			hid_report_send[0]= REPORT_ID_IO;
			hid_report_send[1]= 0x99;			// RetErr(ԐM)\L
			hid_report_send[2]= reciveRetErr;
			hid_report_send[3]= 0;
			hid_report_send[4]= 0;
			hid_report_send[5]= 0;		//country code
			hid_report_send[6]= 0;
			hid_report_send[7]= 0;
			hid_report_send[8]= 0;
    
//			lastINTransmission= HIDTxPacket(HID_EP, (BYTE*)hid_report_send, 0x09);
		// Report Deescriptor Report CountlƂ̊֌W 0x09 ƃ_ HID_INT_IN_EP_SIZE(=64) OK
		// Report Deescriptor ̓͂Əo͂BufferTCYႤlɂ΂悢̂HHH ͖ 2010/07/01()
			lastINTransmission= HIDTxPacket(HID_EP, (BYTE*)hid_report_send, HID_INT_IN_EP_SIZE);
//			DEBUG_LED(mLED_3_Toggle();)
			ReSetRetErrOfCodeData();		// L[E}EXR[h̎MFlag reset
			return(EMU_CALL_OK);
		}
	}

	if(!HIDRxHandleBusy(USBOutHandle))			//MfBmF
	{
		// **Note**			2010/06/07()
		// ŏ̃f[^󂯎ȂBŌ̎Mf[^QdɂȂB
		// HIDRxPacket()f[^Buffer擾Oɑ点ƁAUSBʐM͎̃f[^tɂȂB
		// f[^tɂȂƃf[^Bufferɂ͎̃f[^Ă܂B
//		USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&GeneralInOutBuffer,HID_INT_GENE_EP_SIZE);

		//Lookat the data the host sent, to see what kind of application specific command it sent.
		switch(GeneralInOutBuffer[0])	
		{
			case REPORT_ID_IO:
			{
				DEBUG_PRINTF(PrintSCI((ROM BYTE *)("recive => %s\r\n"), (UINT16)(&GeneralInOutBuffer[1]),0);)
				reciveRetErr = SetHidCodeDataToEEPRom((UINT8 *)(&GeneralInOutBuffer[1]));
				SetRetErrOfCodeData();		// L[E}EXR[h̎MFlag set
//				DEBUG_LED(mLED_4_Toggle();)
				break;
			}
			case REPORT_ID_KEY:
			{
				break;
			}
		}

		USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&GeneralInOutBuffer,HID_INT_GENE_EP_SIZE);
		return(EMU_CALL_OK);
	}

	return(EMU_CALL_FAILED);
}

// *************************************************************
// ėpPCM	󖢊 2010/06/21() ?????????????????
// *************************************************************
Emulate_Error_t SendPalams(void)
{
	SwitchReturn_t		ret;
	Emulate_Error_t		retErr;

	return(EMU_CALL_OK);
//****************************************************************************
//	ȉ点ȂB
//****************************************************************************

	ret = Switch1IsPressed();
	if( ret == SWITCH_EDGE_ON)
	{
		if(!HIDTxHandleBusy(lastINTransmission))			// MfBmF
		{
			hid_report_send[0]= REPORT_ID_IO;
			hid_report_send[1]= 1;
			hid_report_send[2]= 2;
			hid_report_send[3]= 3;
			hid_report_send[4]= 0;
			hid_report_send[5]= 0;		//country code
			hid_report_send[6]= 0;
			hid_report_send[7]= 0;
			hid_report_send[8]= 0;

			lastINTransmission= HIDTxPacket(HID_EP, (BYTE*)hid_report_send, 0x09);
			return(EMU_CALL_OK);
		}
	}
	return(EMU_CALL_FAILED);
}


/**
****************************************************************************
* XCb`SW1
****************************************************************************
	*/
SwitchReturn_t Switch1IsPressed(void)
{
	BYTE NowSW;

	NowSW=SW1;
	if( NowSW != old_SW1)
	{
		old_SW1 = NowSW;			// Save new value
		if(NowSW == 0)				// If pressed
		{
			return(SWITCH_EDGE_ON);		// ON Edge
		}
	}
	else if(NowSW == 0)
	{
		return(SWITCH_ALWAYS_ON);		// 펞ON
	}

	return(SWITCH_ALWAYS_OFF);			// Was not pressed 	펞OFF
}

/**
****************************************************************************
* tbgXCb` `^O
****************************************************************************
	*/
// scan times = 0.1msec (about) * 10 = 1msec (chattering removal time)
#define FOOTSW_SCAN_COUNTES		10

BYTE	FSW1_ChattRemoval, FSW2_ChattRemoval, FSW3_ChattRemoval, FSW4_ChattRemoval;
// `^OJE^
UINT8	FSW1_Cnt, FSW2_Cnt, FSW3_Cnt, FSW4_Cnt;

/**
	`^OXCb` 
	2010/08/11()
	*/
void ChattRemovalInit(void)
{
	FSW1_ChattRemoval = 1;
	FSW2_ChattRemoval = 1;
	FSW3_ChattRemoval = 1;
	FSW4_ChattRemoval = 1;
}

void GetFootSwInfo(FootSwitch_t fswNo)
{
	BYTE	*footInfo;
	UINT8	*fswCnt;
	BYTE	NowFSW;

	switch(fswNo)
	{
		case FOOT_SWITCH1: { footInfo=&FSW1_ChattRemoval; fswCnt=&FSW1_Cnt; NowFSW=FSW_1; break; }
		case FOOT_SWITCH2: { footInfo=&FSW2_ChattRemoval; fswCnt=&FSW2_Cnt; NowFSW=FSW_2; break; }
		case FOOT_SWITCH3: { footInfo=&FSW3_ChattRemoval; fswCnt=&FSW3_Cnt; NowFSW=FSW_3; break; }
		case FOOT_SWITCH4: { footInfo=&FSW4_ChattRemoval; fswCnt=&FSW4_Cnt; NowFSW=FSW_4; break; }
		default:
		{
			break;
		}
	}

	if( *footInfo != NowFSW)
	{
		(*fswCnt)++;
		if(*fswCnt > FOOTSW_SCAN_COUNTES)
		{
			*footInfo = NowFSW;
			*fswCnt = 0;
		}
	}
	else
	{
		*fswCnt = 0;
	}
}

/**
****************************************************************************
* tbgXCb`
*  ɉꂽꍇAGbW͌oΉsB	2010/07/01()
* 		: tbgXCb`ԍi[
* ߂l	: tbgXCb`ƃ^CAbv
****************************************************************************
	*/
SwitchReturn_t FootSwIsPressed(FootSwitch_t *footNo)
{
	BYTE NowSW;

	*footNo = FOOT_SWITCH_NON;
	if( timer1_UpFlag)
	{
		timer1_UpFlag = FALSE;
		return(SWITCH_TIME_UP);
	}

#if defined(ENABLE_FSW_1_DETECT)
	GetFootSwInfo(FOOT_SWITCH1);
	NowSW = FSW1_ChattRemoval;
	if( NowSW != old_FSW1)
	{
		old_FSW1 = NowSW;			// Save new value
		if(NowSW == 0)				// If pressed
		{
			*footNo = FOOT_SWITCH1;
			return(SWITCH_EDGE_ON);		// ON Edge
		}
	}
	else if(NowSW == 0)
	{
		*footNo = FOOT_SWITCH1;
	}
#endif		// ENABLE_FSW_1_DETECT

#if defined(ENABLE_FSW_2_DETECT)
	GetFootSwInfo(FOOT_SWITCH2);
	NowSW=FSW2_ChattRemoval;
	if( NowSW != old_FSW2)
	{
		old_FSW2 = NowSW;			// Save new value
		if(NowSW == 0)				// If pressed
		{
//			DEBUG_LED(mLED_4_Toggle();)
			*footNo = FOOT_SWITCH2;
			return(SWITCH_EDGE_ON);		// ON Edge
		}
	}
	else if(NowSW == 0)
	{
		*footNo = FOOT_SWITCH2;
	}
#endif		// ENABLE_FSW_2_DETECT

#if defined(ENABLE_FSW_3_DETECT)
	GetFootSwInfo(FOOT_SWITCH3);
	NowSW=FSW3_ChattRemoval;
	if( NowSW != old_FSW3)
	{
		old_FSW3 = NowSW;			// Save new value
		if(NowSW == 0)				// If pressed
		{
			*footNo = FOOT_SWITCH3;
			return(SWITCH_EDGE_ON);		// ON Edge
		}
	}
	else if(NowSW == 0)
	{
		*footNo = FOOT_SWITCH3;
	}
#endif		// ENABLE_FSW_3_DETECT

#if defined(ENABLE_FSW_4_DETECT)
	GetFootSwInfo(FOOT_SWITCH4);
	NowSW=FSW4_ChattRemoval;
	if( NowSW != old_FSW4)
	{
		old_FSW4 = NowSW;			// Save new value
		if(NowSW == 0)				// If pressed
		{
			*footNo = FOOT_SWITCH4;
			return(SWITCH_EDGE_ON);		// ON Edge
		}
	}
	else if(NowSW == 0)
	{
		*footNo = FOOT_SWITCH4;
	}
#endif		// ENABLE_FSW_4_DETECT

	if(*footNo != FOOT_SWITCH_NON)
	{
		MONITOR_LED(mLED_4_On();)
		return(SWITCH_ALWAYS_ON);		// 펞ON
	}
	else
	{
		MONITOR_LED(mLED_4_Off();)
		return(SWITCH_ALWAYS_OFF);		// Was not pressed 	펞OFF
	}
}

/**
****************************************************************************
* ^CAbvtOZbg
****************************************************************************
	*/
void Timer1Up(void)
{
	BOOL	ret;
	ret = timer_RepeatSet(1);
	if(ret == FALSE)
	{
		DEBUG_PRINTF(PrintSCI((ROM BYTE *)("Timer1Up\r\n"), 0,0);)
		timer1_UpFlag = TRUE;
		msecTimer1_Stop();
	}
}

void Timer3Up(void)
{
	BOOL	ret;
	ret = timer_RepeatSet(3);
	if(ret == FALSE)
	{
		timer3_UpFlag = TRUE;
		msecTimer3_Stop();
	}
}

/**
****************************************************************************
* ^C}[荞ݏ
****************************************************************************
	*/
#if defined(PIC24F_STARTER_KIT)
void __attribute__((interrupt, no_auto_psv)) _T1Interrupt(void)
{
	Timer1_EvetFlag_Clear();
	Timer1Up();
}

void __attribute__((interrupt, no_auto_psv)) _T2Interrupt(void)
{
//	CallBack_DispLED();
	secTmCounterDecrement();
	Timer2_EvetFlag_Clear();
}

#elif defined(PIC18F2550)

void pic18fT1Interrupt(void)
{
	Timer1_EvetFlag_Clear();
	Timer1Up();
}

void pic18fT3Interrupt(void)
{
	Timer3_EvetFlag_Clear();
	Timer3Up();
}
#endif


/**
****************************************************************************
* gݗČ㓮eXgvO
****************************************************************************
	*/

#if defined(_CIRCUIT_TEST_)

// gݗČ㓮eXg
// PA[^fbvXCb`͂LEDŕ\
// QAtbgXCb`͂LEDŕ\
void cirCuitTest(void)
{
	SwitchReturn_t		ret;
	ret = Switch1IsPressed();
	if( ret == SWITCH_ALWAYS_ON)
	{
		if(DigSW & BIT00)	mRA0_OUT = 0;
		else				mRA0_OUT = 1;
		if(DigSW & BIT01)	mRA1_OUT = 0;
		else				mRA1_OUT = 1;
		if(DigSW & BIT02)	mRA2_OUT = 0;
		else				mRA2_OUT = 1;
		if(DigSW & BIT03)	mRA3_OUT = 0;
		else				mRA3_OUT = 1;
	}
	else
	{
		if(FSW_1)	mRA0_OUT = 1;
		else		mRA0_OUT = 0;
		if(FSW_2)	mRA1_OUT = 1;
		else		mRA1_OUT = 0;
		if(FSW_3)	mRA2_OUT = 1;
		else		mRA2_OUT = 0;
		if(FSW_4)	mRA3_OUT = 1;
		else		mRA3_OUT = 0;
	}
}

#endif			// _CIRCUIT_TEST_

/********************************************************************
 * Function:        void BlinkUSBStatus(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        BlinkUSBStatus turns on and off LEDs 
 *                  corresponding to the USB device state.
 * Note:            mLED macros can be found in HardwareProfile.h
 *                  USBDeviceState is declared and updated in
 *                  usb_device.c.
 *******************************************************************/
void BlinkUSBStatus(void)
{
    static WORD led_count=0;
    
    if(led_count == 0)led_count = 10000U;
    led_count--;

    #define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    #define mLED_Both_On()          {mLED_1_On();mLED_2_On();}
    #define mLED_Only_1_On()        {mLED_1_On();mLED_2_Off();}
    #define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On();}

	DEBUG_LED(mLED_4_Toggle();)

    if(USBSuspendControl == 1)
    {
        if(led_count==0)
        {
            mLED_1_Toggle();
            if(mGetLED_1())
            {
                mLED_2_On();
            }
            else
            {
                mLED_2_Off();
            }
        }//end if
    }
    else
    {
        if(USBDeviceState == DETACHED_STATE)
        {
            mLED_Both_Off();
        }
        else if(USBDeviceState == ATTACHED_STATE)
        {
            mLED_Both_On();
        }
        else if(USBDeviceState == POWERED_STATE)
        {
            mLED_Only_1_On();
        }
        else if(USBDeviceState == DEFAULT_STATE)
        {
            mLED_Only_2_On();
        }
        else if(USBDeviceState == ADDRESS_STATE)
        {
            if(led_count == 0)
            {
                mLED_1_Toggle();
                mLED_2_Off();
            }//end if
        }
        else if(USBDeviceState == CONFIGURED_STATE)
        {
            if(led_count==0)
            {      
                mLED_1_Toggle();         
                if(mGetLED_1())
                {
                    mLED_2_Off();
                }
                else
                {
                    mLED_2_On();
                }
            }//end if
        }//end if(...)
    }//end if(UCONbits.SUSPND...)

}//end BlinkUSBStatus


// ******************************************************************************************************
// ************** USB Callback Functions ****************************************************************
// ******************************************************************************************************
// The USB firmware stack will call the callback functions USBCBxxx() in response to certain USB related
// events.  For example, if the host PC is powering down, it will stop sending out Start of Frame (SOF)
// packets to your device.  In response to this, all USB devices are supposed to decrease their power
// consumption from the USB Vbus to <2.5mA each.  The USB module detects this condition (which according
// to the USB specifications is 3+ms of no bus activity/SOF packets) and then calls the USBCBSuspend()
// function.  You should modify these callback functions to take appropriate actions for each of these
// conditions.  For example, in the USBCBSuspend(), you may wish to add code that will decrease power
// consumption from Vbus to <2.5mA (such as by clock switching, turning off LEDs, putting the
// microcontroller to sleep, etc.).  Then, in the USBCBWakeFromSuspend() function, you may then wish to
// add code that undoes the power saving things done in the USBCBSuspend() function.

// The USBCBSendResume() function is special, in that the USB stack will not automatically call this
// function.  This function is meant to be called from the application firmware instead.  See the
// additional comments near the function.

/******************************************************************************
 * Function:        void USBCBSuspend(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        Call back that is invoked when a USB suspend is detected
 * Note:            None
 *****************************************************************************/
void USBCBSuspend(void)
{
	//Example power saving code.  Insert appropriate code here for the desired
	//application behavior.  If the microcontroller will be put to sleep, a
	//process similar to that shown below may be used:
	
	//ConfigureIOPinsForLowPower();
	//SaveStateOfAllInterruptEnableBits();
	//DisableAllInterruptEnableBits();
	//EnableOnlyTheInterruptsWhichWillBeUsedToWakeTheMicro();	//should enable at least USBActivityIF as a wake source
	//Sleep();
	//RestoreStateOfAllPreviouslySavedInterruptEnableBits();	//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.
	//RestoreIOPinsToNormal();									//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.

	//IMPORTANT NOTE: Do not clear the USBActivityIF (ACTVIF) bit here.  This bit is 
	//cleared inside the usb_device.c file.  Clearing USBActivityIF here will cause 
	//things to not work as intended.	
	

    #if defined(__C30__)
    #if 0
        U1EIR = 0xFFFF;
        U1IR = 0xFFFF;
        U1OTGIR = 0xFFFF;
        IFS5bits.USB1IF = 0;
        IEC5bits.USB1IE = 1;
        U1OTGIEbits.ACTVIE = 1;
        U1OTGIRbits.ACTVIF = 1;
        Sleep();
    #endif
    #endif
}


/******************************************************************************
 * Function:        void _USB1Interrupt(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        This function is called when the USB interrupt bit is set
 *					In this example the interrupt is only used when the device
 *					goes to sleep when it receives a USB suspend command
 * Note:            None
 *****************************************************************************/
#if 0
void __attribute__ ((interrupt)) _USB1Interrupt(void)
{
    #if !defined(self_powered)
        if(U1OTGIRbits.ACTVIF)
        {       
            IEC5bits.USB1IE = 0;
            U1OTGIEbits.ACTVIE = 0;
            IFS5bits.USB1IF = 0;
        
            //USBClearInterruptFlag(USBActivityIFReg,USBActivityIFBitNum);
            USBClearInterruptFlag(USBIdleIFReg,USBIdleIFBitNum);
            //USBSuspendControl = 0;
        }
    #endif
}
#endif

/******************************************************************************
 * Function:        void USBCBWakeFromSuspend(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        The host may put USB peripheral devices in low power
 *					suspend mode (by "sending" 3+ms of idle).  Once in suspend
 *					mode, the host may wake the device back up by sending non-
 *					idle state signalling.
 *					
 *					This call back is invoked when a wakeup from USB suspend 
 *					is detected.
 * Note:            None
 *****************************************************************************/
void USBCBWakeFromSuspend(void)
{
	// If clock switching or other power savings measures were taken when
	// executing the USBCBSuspend() function, now would be a good time to
	// switch back to normal full power run mode conditions.  The host allows
	// a few milliseconds of wakeup time, after which the device must be 
	// fully back to normal, and capable of receiving and processing USB
	// packets.  In order to do this, the USB module must receive proper
	// clocking (IE: 48MHz clock must be available to SIE for full speed USB
	// operation).
}

/********************************************************************
 * Function:        void USBCB_SOF_Handler(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        The USB host sends out a SOF packet to full-speed
 *                  devices every 1 ms. This interrupt may be useful
 *                  for isochronous pipes. End designers should
 *                  implement callback routine as necessary.
 * Note:            None
 *******************************************************************/
void USBCB_SOF_Handler(void)
{
    // No need to clear UIRbits.SOFIF to 0 here.
    // Callback caller is already doing that.
}

/*******************************************************************
 * Function:        void USBCBErrorHandler(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        The purpose of this callback is mainly for
 *                  debugging during development. Check UEIR to see
 *                  which error causes the interrupt.
 * Note:            None
 *******************************************************************/
void USBCBErrorHandler(void)
{
    // No need to clear UEIR to 0 here.
    // Callback caller is already doing that.

	// Typically, user firmware does not need to do anything special
	// if a USB error occurs.  For example, if the host sends an OUT
	// packet to your device, but the packet gets corrupted (ex:
	// because of a bad connection, or the user unplugs the
	// USB cable during the transmission) this will typically set
	// one or more USB error interrupt flags.  Nothing specific
	// needs to be done however, since the SIE will automatically
	// send a "NAK" packet to the host.  In response to this, the
	// host will normally retry to send the packet again, and no
	// data loss occurs.  The system will typically recover
	// automatically, without the need for application firmware
	// intervention.
	
	// Nevertheless, this callback function is provided, such as
	// for debugging purposes.
}


/*******************************************************************
 * Function:        void USBCBCheckOtherReq(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        When SETUP packets arrive from the host, some
 * 					firmware must process the request and respond
 *					appropriately to fulfill the request.  Some of
 *					the SETUP packets will be for standard
 *					USB "chapter 9" (as in, fulfilling chapter 9 of
 *					the official USB specifications) requests, while
 *					others may be specific to the USB device class
 *					that is being implemented.  For example, a HID
 *					class device needs to be able to respond to
 *					"GET REPORT" type of requests.  This
 *					is not a standard USB chapter 9 request, and 
 *					therefore not handled by usb_device.c.  Instead
 *					this request should be handled by class specific 
 *					firmware, such as that contained in usb_function_hid.c.
 * Note:            None
 *******************************************************************/
void USBCBCheckOtherReq(void)
{
	/**
		C:\Microchip Solutions\Microchip\Usb\HID Device Driver\usb_function_hid.c
    	*/
    USBCheckHIDRequest();
}//end


/*******************************************************************
 * Function:        void USBCBStdSetDscHandler(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        The USBCBStdSetDscHandler() callback function is
 *					called when a SETUP, bRequest: SET_DESCRIPTOR request
 *					arrives.  Typically SET_DESCRIPTOR requests are
 *					not used in most applications, and it is
 *					optional to support this type of request.
 * Note:            None
 *******************************************************************/
void USBCBStdSetDscHandler(void)
{
    // Must claim session ownership if supporting this request
}//end


/*******************************************************************
 * Function:        void USBCBInitEP(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        This function is called when the device becomes
 *                  initialized, which occurs after the host sends a
 * 					SET_CONFIGURATION (wValue not = 0) request.  This 
 *					callback function should initialize the endpoints 
 *					for the device's usage according to the current 
 *					configuration.
 * Note:            None
 *******************************************************************/
void USBCBInitEP(void)
{
	/**
		C:\FootMouseKey\system\Usb\usb_device.c
		*/
    //enable the HID endpoint
    // mouse
//    USBEnableEndpoint(HID_EP, USB_IN_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);

/**
	C:\FootMouseKey\user\Keyboard.c
	*/
	USBEnableEndpoint(HID_EP,USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
	USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&GeneralInOutBuffer,HID_INT_GENE_EP_SIZE);

}

/********************************************************************
 * Function:        void USBCBSendResume(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        
 *******************************************************************/
void USBCBSendResume(void)
{
    static WORD delay_count;
    
    USBResumeControl = 1;                // Start RESUME signaling
    
    delay_count = 1800U;                // Set RESUME line for 1-13 ms
    do
    {
        delay_count--;
    } while(delay_count);

    USBResumeControl = 0;
}


/*******************************************************************
 * Function:        BOOL USER_USB_CALLBACK_EVENT_HANDLER(
 *                        USB_EVENT event, void *pdata, WORD size)
 * PreCondition:    None
 * Input:           USB_EVENT event - the type of event
 *                  void *pdata - pointer to the event data
 *                  WORD size - size of the event data
 * Output:          None
 * Side Effects:    None
 * Overview:        This function is called from the USB stack to
 *                  notify a user application that a USB event
 *                  occured.  This callback is in interrupt context
 *                  when the USB_INTERRUPT option is selected.
 * Note:            None
 *******************************************************************/
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
	switch(event)
	{
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        case EVENT_TRANSFER:
            Nop();
            break;
        default:
            break;
    }      
    return TRUE; 
}

// *****************************************************************************
// ************** USB Class Specific Callback Function(s) **********************
// *****************************************************************************

/********************************************************************
 * Function:        void USBHIDCBSetReportHandler(void)
 * PreCondition:    None
 * Input:           None
 * Output:          None
 * Side Effects:    None
 * Overview:        USBHIDCBSetReportHandler() is used to respond to
 *					the HID device class specific SET_REPORT control
 *					transfer request (starts with SETUP packet on EP0 OUT).  
 * Note:            
 *******************************************************************/
void USBHIDCBSetReportHandler(void)
{
	//Prepare to receive the keyboard LED state data through a SET_REPORT
	//control transfer on endpoint 0.  The host should only send 1 byte,
	//since this is all that the report descriptor allows it to send.
/**
	C:\FootMouseKey\system\Include\Usb\usb_device.h
	#define USBEP0Receive(dest,size,function)  {outPipes[0].pDst.bRam = dest;
												outPipes[0].wCount.Val = size;
												outPipes[0].pFunc = function;
												outPipes[0].info.bits.busy = 1; }
	*/
	USBEP0Receive((BYTE*)&CtrlTrfData, USB_EP0_BUFF_SIZE, USBHIDCBSetReportComplete);
}

//Secondary callback function that gets called when the above
//control transfer completes for the USBHIDCBSetReportHandler()
void USBHIDCBSetReportComplete(void)
{
	//1 byte of LED state data should now be in the CtrlTrfData buffer.

	//Num Lock LED state is in Bit0.
	DEBUG_LED(mLED_3_Toggle();)
	if(CtrlTrfData[0] & 0x01)	//Make LED1 and LED2 match Num Lock state.
	{
		mLED_1_On();
		mLED_2_On();
	}
	else
	{
		mLED_1_Off();
		mLED_2_Off();			
	}
	
	//Stop toggling the LEDs, so you can temporily see the Num lock LED state instead.
	//Once the CountdownTimerToShowUSBStatusOnLEDs reaches 0, the LEDs will go back to showing USB state instead.
	BlinkStatusValid = FALSE;	
	CountdownTimerToShowUSBStatusOnLEDs = 140000; 
}

#endif			// USBMOUSE_KEY_C
