/**********************************************************************
***********************************************************************
*	Processor	: PIC18F2550
*	File Name   : sci.c
*	Summary		: VAʐM(RS232C)
*
*	History:
*	Date		Designer	Description
*	07/23/2010	DachsTulip	Created	
*
*	Copyright (c) 2010 by DachsTulip
***********************************************************************
***********************************************************************/

#include	<stdio.h>
#include	<stdarg.h>
#include	<string.h>
#include	<ctype.h>

#include	"picSio.h"
//				|-> #include	"mytype.h"
// RS232CVAʐMSĂLEɂRpCXCb`
// 				|-> #define __SIO_ENABLE__


#if defined(__SIO_ENABLE__)	// VAʐM\[XLɂ`

/**
***********************************************************************
* debug Definitions
************************************************************************
	*/
 #define _ERROR_PRINTF_
#if defined(_ERROR_PRINTF_)
	#if defined(__SIO_ENABLE__)
		#define ERROR_PRINTF(__dbg) __dbg
	#else
		#define ERROR_PRINTF(__dbg)
	#endif
#else
	#define ERROR_PRINTF(__dbg)
#endif

// #define _DEBUG_LED_
#if defined(_DEBUG_LED_)
	#define DEBUG_LED(__dbg) __dbg
#else
	#define DEBUG_LED(__dbg)
#endif


/**
***********************************************************************
* buffer definition
************************************************************************
*/

/**
	\src\sio.h
	#define		SCI_BUFF_SIZE	1024
	#define		SCI_BUFF_SIZE	200
	*/
static UINT8	rx0_buff[SCI_BUFF_SIZE];
static UINT8	tx0_buff[SCI_BUFF_SIZE];
static SIO_FIFO rx0_fifo;
static SIO_FIFO tx0_fifo;

static SIO_Info_t	sio_info[SCI_CHNEND];


/** DECLARATIONS ***************************************************/
#pragma code

/************************************/
/*	VA|[g			*/
/************************************/

void sci_initial(void)
{
	UINT16		i;
	for(i=SCI_CHN0; i<SCI_CHNEND; i++)
	{
		sio_info[i].RunStop = FALSE;
	}

//	sci_start(SCI_CHN0, SERIAL_BPS, 8, 0, 1);			/*	SCI1 19200bps	*/
	sci_start(SCI_CHN0, SERIAL_HIGH_BPS, 8, 0, 1);		/*	SCI1 115200bps	*/
}


BOOL  sci_cts_dsr_switch_confirm(void)
{
	return(FALSE);							/* cts rts  */
}

static Sio_Error_t  sci_cts_dsr_check(SIO_Channel_t chn, BOOL RtsDtr)
{
	if(sci_cts_dsr_switch_confirm())
	{
		if(chn == SCI_CHN0)		/* Because it becomes CTS and RTS for channel1 this Ineffective setting */
		{
//			return(SIO_CALL_FAILED);
		}
	}

	return( SIO_CALL_OK);
}

/**
	2008/05/20
	Note:UINT16 bps -> uint32 bps		115200bps(0x1C200)  exceed 2 byte
	C:\Program Files\Microchip\MPLAB C30\support\PIC24F\h\p24FJ256GB106.h
	*/
Sio_Error_t  sci_start( SIO_Channel_t	chn,
						UINT32			bps,			/*	4800~	*/
						UINT8			bit,			/*	8 or 7	*/
						UINT8			parity,			/*	0:NON 1:EVN 2:ODD	*/
						UINT8			stopBit )		/*	1 or 2	*/
{

		/* ring buffer and port setting */
	SIO_FIFO		*rx_ring, *tx_ring;

	if(	   ((chn == SCI_CHN0) && (bit     == 8))
		&& ((0 <= parity )    && (parity  <= 3))
		&& ((1 <= stopBit)    && (stopBit <= 2)) )
	{
		/* params OK */
	}
	else
	{
		return(SIO_CALL_FAILED);
	}

	if(sci_cts_dsr_check( chn, TRUE) != SIO_CALL_OK)		/* CTS/RTS LEH */
	{
		return(SIO_CALL_FAILED);
	}

		/* clock and buffer setting */
	{
		if(chn == SCI_CHN0)
		{
//			STB.CR3.BIT._SCI0 = 0;				/* MSTP11(B3)FSCI_0 clock  */
			rx_ring = &rx0_fifo; rx_ring->buff = rx0_buff; 
			tx_ring = &tx0_fifo; tx_ring->buff = tx0_buff; 
		}

		rx_ring->length = 0; rx_ring->read = 0; rx_ring->write = 0;	/*	RX FIFO	*/
		tx_ring->length = 0; tx_ring->read = 0; tx_ring->write = 0;	/*	TX FIFO	*/
	}

		/* ݋֎~ setteing */
	{
		if(chn == SCI_CHN0)
		{
			UINT16		intLevel;
													/* InterruptLevel=7 */
			if(chn == SCI_CHN0)	{  intLevel = 0x7000; }
			/**
				C:\Program Files\Microchip\MPLAB C30\support\PIC24F\h\p24FJ256GB106.h
				C:\Program Files\Microchip\MCC18\h\p18f2550.h
				*/

		#if defined(PIC24F_STARTER_KIT)
			IPC3bits.U1TXIP = 4;			// MݗD惌xݒ 4??
			IPC2bits.U1RXIP = 4;
		#elif defined(PIC18F2550)
			IPR1bits.TXIP = 1;				// EUSART Transmit Interrupt Priority bit	1:High priority
			IPR1bits.RCIP = 1;				// EUSART Receive Interrupt Priority bit
		#endif
		}
	}

	/*	VA[hWX^iSCSMRj  CKS 00FPӃNbN */
#if defined(PIC24F_STARTER_KIT)
	{
		UINT8		smr = 0;
		UINT8		stp = 0;
		if ( 1 == parity )			{ smr |= 0x01; }
		else if ( 2 == parity )		{ smr |= 0x02; }
		if ( 2 == stopBit )			{ stp |= 0x01; }

		if(chn == SCI_CHN0)
		{
			U1MODEbits.PDSEL	= smr;		/* VA[hWX^ B1B0=00FPӃNbN */
			U1MODEbits.STSEL	= stp;			/* stop bit set */

			if( bps != SERIAL_HIGH_BPS)
			{
				U1BRG				= ((PCCLOCK / 16) / bps) - 1;
			}
			else
			{
				// SERIAL_HIGH_BPS(115200bps) Ȃ "-1"8sec Ȃ 9sec
				// vZl 8.68sec łB߂̂͂Ȃ̕ƂȂB
				U1BRG				= ((PCCLOCK / 16) / bps);
			}
		}
	}
#elif defined(PIC18F2550)
	{
		if(chn == SCI_CHN0)
		{
			TXSTAbits.SYNC = 0;				// 0 = Asynchronous mode
			TXSTAbits.BRGH = 1;				// High Baud Rate Select bit(Asynchronous mode: 1 = High speed)
			BAUDCONbits.BRG16 = 1;			// 16-Bit Baud Rate Register Enable bit
											// 		(1 = 16-bit Baud Rate Generator - SPBRGH and SPBRG)
//			TXSTAbits.TX9 = 0;				// 0 = Selects 8-bit transmission
			TXSTAbits.BRGH = 1;				// High Baud Rate Select bit(1 = High speed)
			{
				UINT16	SpbSetting;
					// #pragma config FOSC = HSPLL_HS(VXeNbN48MH) -> HS(VXeNbN20MH)
					// LݒŁAVÂPhbg~1/2.4{ZłB
				SpbSetting = ((CLOCK_FREQ / 4) / bps) - 1;	// FOSC/[4 (n + 1)]
				SPBRGH = (UINT8)(SpbSetting>>8);
				SPBRG = (UINT8)(SpbSetting & 0x00FF);
			}
		}
	}
#endif
		/*	Xe[^XNA*/
	{
		if(chn == SCI_CHN0)
		{
		#if defined(PIC24F_STARTER_KIT)
//			UINT8 dummy = SCI_CHN->SCSSR.BYTE;
			UINT16 dummy = U1STA;
		#elif defined(PIC18F2550)
		#endif
		}
	}
		/* port setteing */
	{
		if(chn == SCI_CHN0)
		{
		#if defined(PIC24F_STARTER_KIT)
			U1MODEbits.UARTEN = 1;		/* UART1L */
			// assign U1TX To Pin RP3
			RPOR1bits.RP3R = 3;
		#elif defined(PIC18F2550)
			mInitEUSART();
			RCSTAbits.SPEN = 1;			// Serial Port Enable bit
							// (1 = Serial port enabled (configures RX/DT and TX/CK pins as serial port pins))
			RCSTAbits.CREN = 1;			// Continuous Receive Enable bit(Asynchronous mode : 1 = Enables receiver)
//			TXSTAbits.TRMT			// Transmit Shift Register Status bit(1 = TSR empty)
//			BAUDCONbits.RXDTP			// Received Data Polarity Select bit
		#endif
		}
	}

	/*	EM݋	*/
	{
		if(chn == SCI_CHN0)
		{
		#if defined(PIC24F_STARTER_KIT)
			// MMVtgWX^ɓ]AMBufferɂȂݔ
			U1STAbits.UTXISEL1 = 0;
			U1STAbits.UTXISEL0 = 1;
			IEC0bits.U1TXIE = 1;			// Enable Transmit Interrupt
			IEC0bits.U1RXIE = 1;			// Enable Receive Interrupt
		#elif defined(PIC18F2550)
			PIE1bits.TXIE = 0;				// disable (Enable Transmit Interrupt)
			PIE1bits.RCIE = 1;				// Enable Receive Interrupt
//			TSR(Transmit Shift Reg) Register LɂB2010/05/22(y) 16:55:38
			TXSTAbits.TXEN = 1;				// Transmit Shift Reg Enable bit(1 = Transmit enabled)
		#endif
		}
	}

	sio_info[chn].RunStop = TRUE;
	return( SIO_CALL_OK);
}


void sci_stop( SIO_Channel_t chn)
{
	if(chn == SCI_CHN0) {}
	else				{ return; }

	if(chn == SCI_CHN0)
	{
	#if defined(PIC24F_STARTER_KIT)
		IEC0bits.U1TXIE = 0;			// Enable Transmit Interrupt
		IEC0bits.U1RXIE = 0;			// Enable Receive Interrupt
	#elif defined(PIC18F2550)
		PIE1bits.TXIE = 0;				// Enable Transmit Interrupt
		PIE1bits.RCIE = 0;				// Enable Receive Interrupt
//			TSR(Transmit Shift Reg) Register 𖳌ɂB2010/05/22(y) 16:55:41
		TXSTAbits.TXEN = 0;				// Transmit Enable bit(1 = Transmit enabled)
	#endif
	}

	/* 2008/05/27 clock~́A~ɂ背WX^ɃANZXoȂȂ\AŌɋLq */
	if(chn == SCI_CHN0)
	{
	#if defined(PIC24F_STARTER_KIT)
		U1MODEbits.UARTEN = 0;		/*  TXD0 -> 	 RXD0 ->  */
//		STB.CR3.BIT._SCI0 = 1;				/* MSTP11(B3)FSCI_0 clock ~ */
	#elif defined(PIC18F2550)

	#endif
	}

	sio_info[chn].RunStop = FALSE;
}


/************************************/
/*	sio 擾					*/
/************************************/

static BOOL  sio_RunStop_info(SIO_Channel_t chn)
{
	return(sio_info[chn].RunStop);
}

UINT16 sci_tx_remainderLength(void)
{
	return(tx0_fifo.length);
}

/************************************/
/*	Pi[						*/
/************************************/
Sio_Error_t  sci_putc(SIO_Channel_t chn, UINT8 data)
{
	Sio_Error_t		result = SIO_CALL_FAILED;
	SIO_FIFO		*txfifo;

	if(sio_RunStop_info(chn) != TRUE)
	{
		return(SIO_CALL_FAILED);
	}

	if(sci_cts_dsr_check( chn, FALSE) != SIO_CALL_OK)
	{
		return(SIO_CALL_FAILED);
	}

	if(chn == SCI_CHN0)	{ txfifo = &tx0_fifo; }
	else				{ return(SIO_CALL_FAILED); }

	disable_interrupt();
	{
//		SCI_CHN->SCSCR.BIT.TIE = 0;		/*	M݋֎~	*/
							/* 2008/5/22 Odisable_interruptĂ̂ Comment Out */
		if ( txfifo->length < SCI_BUFF_SIZE ) 
		{
			txfifo->buff[txfifo->write] = data;

			txfifo->length++;
			txfifo->write++;
			if ( txfifo->write >= SCI_BUFF_SIZE ) 
			{
				txfifo->write = 0;
			}

			if(chn == SCI_CHN0)
			{
			#if defined(PIC24F_STARTER_KIT)
				U1STAbits.UTXEN = 1;			/* MBit */
			#elif defined(PIC18F2550)
				PIE1bits.TXIE = 1;				// Enable Transmit Interrupt
//				TXSTAbits.TXEN = 1;				// Transmit Enable bit(1 = Transmit enabled)
												// 2010/05/22(y) 16:54:40@RgAEg
			#endif
			}
			result = SIO_CALL_OK;
		}
		else 
		{
			result = SIO_SCI_BUFFER_OVER;		/* buffer size over */
		}
//		SCI_CHN->SCSCR.BIT.TIE = 1;		/*	M݋	*/
	}
	enable_interrupt();

	return (result);
}


/************************************/
/*	P擾						*/
/************************************/
Sio_Error_t  sci_getc(SIO_Channel_t chn, UINT8 *rxdt)
{
	Sio_Error_t		result = SIO_CALL_FAILED;
	SIO_FIFO		*rx_ring;

	if(sio_RunStop_info(chn) != TRUE)
	{
		return(SIO_CALL_FAILED);
	}

	if(chn == SCI_CHN0)	{ rx_ring = &rx0_fifo; }
	else				{ return(SIO_CALL_FAILED); }

	disable_interrupt();
	{
							/* 2008/5/22 Odisable_interruptĂ̂ Comment Out */

		if ( 0 != rx_ring->length  ) 
		{
			*rxdt = rx_ring->buff[rx_ring->read];

			rx_ring->length--;
			rx_ring->read++;
			if ( rx_ring->read >= SCI_BUFF_SIZE) 
			{
				rx_ring->read = 0;
			}
			result = SIO_CALL_OK;
		}
	}
	enable_interrupt();

	return( result);
}


/************************************/
/*	MG[荞				*/
/************************************/
#if defined(PIC24F_STARTER_KIT)
// #pragma interrupt (int_eri0)
// void int_eri0(void)
void __attribute__((interrupt, no_auto_psv)) _U1ErrInterrupt(void)
{
	     if(U1STAbits.PERR){ U1STAbits.PERR = 0; }
	else if(U1STAbits.FERR){ U1STAbits.FERR = 0; }		// Framing Error bit
	else if(U1STAbits.OERR){ U1STAbits.OERR = 0; }		// Overrun Error bit
}
#elif defined(PIC18F2550)

/**
	C:\Program Files\Microchip\MCC18\h\p18f2550.h
	*/
void EUSART_ErrInterrupt(void)
{
	     if(RCSTAbits.FERR){ RCSTAbits.FERR = 0; }		// Framing Error bit
	else if(RCSTAbits.OERR){ RCSTAbits.OERR = 0; }		// Overrun Error bit
}

#endif


/************************************/
/*	M荞					*/
/************************************/
#define SCI_RDRF_RESET		~0x40

Sio_Error_t rxi_fifo_input(SIO_FIFO *rxfifo, char rxdt)
{
	if( rxfifo->length < SCI_BUFF_SIZE)		// McBufferSize𒴂ĂȂH
	{
		if(rxdt == kCRcode)	
		{	// 2008/07/14 ݑ΍ COM3{[hƃs[g@\삵Ȃ(s)̂Œǉ
//			sio_repeat_hexdump_Flag_reset();
		}

//		if(sio_command_busyFlag_check() && (rxfifo->length==0) && rxdt == kCRcode)
		if((rxfifo->length==0) && (rxdt == kCRcode))
		{
			/**
				R}h́AG^[L[󂯕tȂB
				󂯕tƒ̏ꍇAx̒s
				*/
			return(SIO_CALL_CRCODE);
		}

		{
			rxfifo->buff[rxfifo->write]= rxdt;
			rxfifo->length++;
			rxfifo->write++;
			if( rxfifo->write >= SCI_BUFF_SIZE)
			{
				rxfifo->write= 0;
			}
			return( SIO_CALL_OK);
		}
	}
	return(SIO_SCI_BUFFER_OVER);
}

/************************************/
/*	M荞					*/
/************************************/
/**
	݊֌W
	C:\Program Files\Microchip\MPLAB C30\docs\MPLAB_ALU_PIC24_dsPIC_UG_51317G.pdf
	*/
// #pragma interrupt (int_rxi0)
// void int_rxi0(void)
#if defined(PIC24F_STARTER_KIT)
void __attribute__((interrupt, no_auto_psv)) _U1RXInterrupt(void)
{
	UINT8	data;

	if(U1STAbits.URXDA)					/* MBufferɃf[^邩H */
	{
		data	= U1RXREG;				/*	M	*/

		if(sci_cts_dsr_check( SCI_CHN0, FALSE) == SIO_CALL_OK)
		{
			rxi_fifo_input( &rx0_fifo, data);
		}
	}
	IFS0bits.U1RXIF = 0;			/*	RDRFNA	*/
}

#elif defined(PIC18F2550)
void EUSART_RXInterrupt(void)
{
	UINT8	data;
	if(PIR1bits.RCIF)					/* MBufferɃf[^邩H */
	{
	/**
		C:\Program Files\Microchip\MCC18\h\p18f2550.h
		*/
		data	= RCREG;				/*	M	*/
//		DEBUG_LED(mLED_4_Toggle();)

		if(sci_cts_dsr_check( SCI_CHN0, FALSE) == SIO_CALL_OK)
		{
			rxi_fifo_input( &rx0_fifo, data);
		}
	}
	// Framing Error bit
	if(RCSTAbits.FERR)
	{
		// (can be updated by reading RCREG register and receiving next valid byte
	}
	// 2010/06/22()
	// Overrun Error bit (can be cleared by clearing bit CREN)
	if(RCSTAbits.OERR)
	{
		ERROR_PRINTF(PrintSCI((ROM BYTE *)("oerr RCSTA:%x\r\n"), (UINT16)RCSTA, 0);)
		RCSTAbits.CREN = 0;				// Continuous Receive Enable bit
		while(RCSTAbits.OERR);
		RCSTAbits.CREN = 1;				// Continuous Receive Enable bit
	}
}
#endif

/************************************/
/*	M荞					*/
/************************************/
static void txi_fifo_output(SIO_Channel_t chn, SIO_FIFO *txfifo)
{
	UINT8	data;

	if(chn == SCI_CHN0)	{ }
	else				{ return; }

	if ( 0 < txfifo->length ) 		// Mc邩H
	{
		data = txfifo->buff[txfifo->read];

		txfifo->length--;
		txfifo->read++;
		if( txfifo->read >= SCI_BUFF_SIZE)
		{
			txfifo->read = 0;
		}

		if(chn == SCI_CHN0)
		{
		#if defined(PIC24F_STARTER_KIT)
			// U1TXIF : MBuffe瑗MShift register(U1TSR)ɕ]"1"ƂȂB
			IFS0bits.U1TXIF = 0;				/* MFlag */
			U1TXREG		= data;					/* PoCgM	*/
		#elif defined(PIC18F2550)
//			PIR1bits.TXIF = 0;					// EUSART Transmit Interrupt Flag bit
			TXREG = data;						/* PoCgM	*/
		#endif
		}
	}
	else 
	{
		if(chn == SCI_CHN0)
		{
		#if defined(PIC24F_STARTER_KIT)
			// U1TXIF : MBuffe瑗MShift register(U1TSR)ɕ]"1"ƂȂB
			IFS0bits.U1TXIF = 0;				/* MFlag */
			U1STAbits.UTXEN = 0;				/* MLBit */
		#elif defined(PIC18F2550)
//			PIR1bits.TXIF = 0;					// EUSART Transmit Interrupt Flag bit
			PIE1bits.TXIE = 0;					// disable (Enable Transmit Interrupt)

//			ȉbitOFFTSR RegisterƂȂ̂ōŌ̃f[^]rŖƂȂB
												// 2010/05/22(y) 16:54:40
//			TXSTAbits.TXEN = 0;					// Transmit Enable bit(0 = Transmit disabled)
		#endif
		}
	}
}

// #pragma interrupt (int_txi0)
// void int_txi0(void)
#if defined(PIC24F_STARTER_KIT)
void __attribute__((interrupt, no_auto_psv)) _U1TXInterrupt(void)
{
	txi_fifo_output( SCI_CHN0, &tx0_fifo);
}

#elif defined(PIC18F2550)
/**
	C:\FootMouseKey\user\mouseKey.c
		|-> Remapped_High_ISR()
			|-> YourHighPriorityISRCode()
				|-> EUSART_TXInterrupt()
	*/
void EUSART_TXInterrupt(void)
{
	txi_fifo_output( SCI_CHN0, &tx0_fifo);
}
#endif

#endif		// #ifdef __SIO_ENABLE__

